from ._dispatch_wrapper import DispatchWrapper


class DrillItem(DispatchWrapper):
    """ A Drill Item found in an engineering log gives information about the dimensions of the drilled borehole.

    The drill item gives information about the diameter and length of the borehole. You can have multiple drill items
    in a borehole if the diameter varies at certain depths. Only a bottom depth is specified: it will range from the
    top of the borehole (at depth 0) to the specified bottom_depth OR if another drill item is present above,
    it will range from the bottom of the other drill to the bottom_depth of the current drill.

    Example
    -------
    >>> log = borehole.log("Well Sketch")
    >>> drill_item = log.drill_item(0)
    >>> drill_item.diameter
    300
    >>> drill_item.comment
    'Drill
    0.00-15.00
    diameter:300'
    """
    @property
    def bottom_depth(self):
        """float: The bottom depth of the drill in current depth units."""
        return self._dispatch.BottomDepth

    @bottom_depth.setter
    def bottom_depth(self, value):
        self._dispatch.BottomDepth = value

    @property
    def diameter(self):
        """float: The diameter of the drill in arbitrary units.

        Note that some other dimensions in an engineering log are
        relative to this measure."""
        return self._dispatch.Diameter

    @diameter.setter
    def diameter(self, value):
        self._dispatch.Diameter = value

    @property
    def comment(self):
        """str: The comment associated with the drill item."""
        return self._dispatch.Comment

    @comment.setter
    def comment(self, value):
        self._dispatch.Comment = value



from ._dispatch_wrapper import DispatchWrapper


class MarkerItem(DispatchWrapper):
    """ A marker that can used to represent formation tops, major/minor contacts, unconformities, etc.

    Example
    -------
    >>> log = borehole.log("Formations")
    >>> marker = log.marker(0)
    >>> marker.name = 'shale'
    """

    @property
    def depth(self):
        """float: The depth of the marker in current depth
        units."""
        return self._dispatch.Depth

    @property
    def name(self):
        """str: The name of the marker."""
        return self._dispatch.Name

    @name.setter
    def name(self, value):
        self._dispatch.Name = value

    @property
    def comment(self):
        """str: The comment for this marker."""
        return self._dispatch.Comment

    @comment.setter
    def comment(self, value):
        self._dispatch.Comment = value

    @property
    def contact(self):
        """str: The name of the contact style to be used.
        
        This must be available in the contact dictionary of the Marker Log this
        marker is a part of.
        """
        return self._dispatch.Contact

    @contact.setter
    def contact(self, value):
        self._dispatch.Contact = value



from ._dispatch_wrapper import DispatchWrapper


class Page(DispatchWrapper):
    """ The Page class manages properties for the document print out.

    Example
    -------
    >>> import wellcad.com
    >>> app = wellcad.com.Application()
    >>> app.new_borehole()
    <wellcad.com._borehole.Borehole object at 0x0000018B3DAF9D30>
    >>> borehole = app.get_active_borehole()
    >>> page = borehole.page
    """

    @property
    def depth_range(self):
        """int : Identify the depth range mode

        The available modes are the following:

        * 0 = depth range not defined (maximum depth range to be printed)
        * 1 = depth range defined by the user
        """
        return self._dispatch.DepthRange

    @depth_range.setter
    def depth_range(self, mode):
        self._dispatch.DepthRange = mode

    @property
    def document_height(self):
        """float: The document height in mm."""
        return self._dispatch.DocumentHeight

    @property
    def document_width(self):
        """float: The document width in mm."""
        return self._dispatch.DocumentWidth

    @document_width.setter
    def document_width(self, value):
        self._dispatch.DocumentWidth = value

    @property
    def nb_of_depth_range(self):
        """double: The number of defined depth ranges."""
        return self._dispatch.NbOfDepthRange

    @property
    def paper_mode(self):
        """int: 0 for page-by-page and 1 for fanfold."""
        return self._dispatch.PaperMode

    @paper_mode.setter
    def paper_mode(self, mode):
        self._dispatch.PaperMode = mode

    @property
    def print_titles_on_top(self):
        """bool: Show the log titles at the top of the printout."""
        return self._dispatch.PrintTitlesOnTop

    @print_titles_on_top.setter
    def print_titles_on_top(self, show):
        self._dispatch.PrintTitlesOnTop = show

    @property
    def print_titles_on_bottom(self):
        """bool: Show the log titles at the bottom of the printout."""
        return self._dispatch.PrintTitlesOnBottom

    @print_titles_on_bottom.setter
    def print_titles_on_bottom(self, show):
        self._dispatch.PrintTitlesOnBottom = show

    @property
    def print_titles_on_bottom_on_each_page(self):
        """bool: Repeat the log titles at the bottom of each printed page."""
        return self._dispatch.PrintTitlesOnBottomOnEachPage

    @print_titles_on_bottom_on_each_page.setter
    def print_titles_on_bottom_on_each_page(self, show):
        self._dispatch.PrintTitlesOnBottomOnEachPage = show

    @property
    def print_titles_on_top_on_each_page(self):
        """bool: Repeat the log titles at the top of each printed page."""
        return self._dispatch.PrintTitlesOnTopOnEachPage

    @print_titles_on_top_on_each_page.setter
    def print_titles_on_top_on_each_page(self, flag):
        self._dispatch.PrintTitlesOnTopOnEachPage = flag

    @property
    def top_margin(self):
        """int: The top margin of the page to print in mm."""
        return self._dispatch.TopMargin

    @top_margin.setter
    def top_margin(self, value):
        self._dispatch.TopMargin = value

    @property
    def bottom_margin(self):
        """int: The bottom margin of the page to print in mm."""
        return self._dispatch.BottomMargin

    @bottom_margin.setter
    def bottom_margin(self, value):
        self._dispatch.BottomMargin = value

    @property
    def left_margin(self):
        """int: The left margin of the page to print in mm."""
        return self._dispatch.LeftMargin

    @left_margin.setter
    def left_margin(self, value):
        self._dispatch.LeftMargin = value

    @property
    def right_margin(self):
        """int: The right margin of the page to print in mm."""
        return self._dispatch.RightMargin

    @right_margin.setter
    def right_margin(self, value):
        self._dispatch.RightMargin = value

    @property
    def numbering(self):
        """int: The page numbering mode

        The available modes are the following:

        * 0 = none
        * 1 = left
        * 2 = right
        * 3 = center
        * 4 = alternating
        """
        return self._dispatch.Numbering

    @numbering.setter
    def numbering(self, mode):
        self._dispatch.Numbering = mode

    @property
    def print_header(self):
        """bool: Option to print the document header or not."""
        return self._dispatch.PrintHeader

    @print_header.setter
    def print_header(self, flag):
        self._dispatch.PrintHeader = flag

    def add_depth_range(self, top, bottom):
        """Adds a new depth range to be printed in current master depth units.
        
        Parameters
        ----------
        top : float
            Top depth of the interval that will be added to the print list.
        bottom : float
            Bottom depth of the interval that will be added to the print list.
        """
        return self._dispatch.AddDepthRange(top, bottom)

    def remove_depth_range(self, index):
        """Remove an entry from the list of depth ranges.
        
        Parameters
        ----------
        index : int
            Zero based index of the entry to be removed from the list.
        """
        return self._dispatch.RemoveDepthRange(index)



from ._application import Application
from ._borehole import Borehole
from ._depth import Depth
from ._header import Header
from ._log import Log
from ._odbc import Odbc
from ._page import Page
from ._title import Title
from ._workspace import Workspace
from ._font import Font
from ._interval_item import IntervalItem
from ._polar_and_rose_box import PolarAndRoseBox
from ._drill_item import DrillItem
from ._equipment_item import EquipmentItem
from ._structure import Structure
from ._litho_bed import LithoBed
from ._cross_section_box import CrossSectionBox
from ._marker_item import MarkerItem
from ._comment_box import CommentBox
from ._stacking_pattern_item import StackingPatternItem
from ._litho_dictionary import LithoDictionary
from ._litho_pattern import LithoPattern
from ._fossil_item import FossilItem



from ._dispatch_wrapper import DispatchWrapper


class IntervalItem(DispatchWrapper):
    """ Stores a value that spans a depth interval.

    Interval items are part of an Interval Log and store a single data value.
    Only numerical values are supported and depth intervals may overlap.

    Example
    -------
    >>> interval_log = borehole.log("Test")
    >>> data = interval_log.interval_item_at_depth(10.5)
    >>> data.value
    23.45
    >>> data.top_depth
    9.0
    """

    @property
    def top_depth(self):
        """float: The top depth of the interval item in current
        depth reference units"""
        return self._dispatch.TopDepth

    @property
    def bottom_depth(self):
        """float: The bottom depth of the interval item in current
        depth reference units"""
        return self._dispatch.BottomDepth

    @property
    def value(self):
        """float: The value of the interval item."""
        return self._dispatch.Value

    @value.setter
    def value(self, value):
        self._dispatch.Value = value





from ._dispatch_wrapper import DispatchWrapper


class CrossSectionBox(DispatchWrapper):
    """ Stores a depth interval over which a cross section from an image log is created.

    CrossSection boxes are part of a CrossSection Log and stores a depth interval.
    Top and bottom depths of the intervals can be the same.

    Example
    -------
    >>> cross_log = borehole.log("Test")
    >>> cross_box = cross_log.insert_new_cross_box(10.5, 12.5)
    >>> data.top_depth
    10.5
    >>> data.top_depth
    12.5
    """

    @property
    def top_depth(self):
        """float : The top depth of the box in current
        depth reference units."""
        return self._dispatch.TopDepth

    @property
    def bottom_depth(self):
        """float : The bottom depth of the box in current
        depth reference units."""
        return self._dispatch.BottomDepth





import pywintypes
import win32com.client.build
from win32com.client.dynamic import CDispatch


class DispatchWrapper:
    """A helper class that wraps a pywin32 dynamic dispatch COM object.

    Inheriting from this does a couple of things that provides useful
    behaviour:
    
    * It overrides ``__new__`` to type check the dynamic dispatch COM object
      that is passed in as a parameter. If it is a genuine
      ``win32com.client.dynamic.CDispatch`` instance, The class will be created
      and the initializer will be called as usual. If the parameter provided
      is not a dynamic dispatch COM object, the object that will be constructed
      will be ``None``.

      This mimics the typical COM behaviour of using ``NULL`` returns as an
      indication of failure.
    * It provides a method of flagging methods at the dispatch level. Simply
      define ``_DISPATCH_METHODS`` at class level as a tuple of method names.
      The constructor will automatically call ``dispatch._FlagAsMethod`` for
      all of these.
    * It provides a technique to flag attributes at the dispatch level. Because
      of the dynamic nature of the dispatch, ``pywin32`` attempts to detect new
      attributes at the time of access. Unfortunately, if the attribute access
      raises an exception, ``pywin32`` assumes it is not a valid attribute and
      raises an ``AttributeError``.
      
      Defining the class attribute ``_DISPATCH_ATTRIBUTES`` forces ``pywin32``
      to register any attributes in the enumeration as actual attributes.

    Parameters
    ----------
    dispatch : win32com.client.dynamic.CDispatch or None
        A valid win32com.client.dynamic.CDispatch instance to wrap, or None.
    
    Attributes
    ----------
    _dispatch : win32com.client.dynamic.CDispatch
        The dispatch COM object this class is wrapping.
    """

    _DISPATCH_METHODS = ()
    _DISPATCH_ATTRIBUTES = ()

    def __new__(cls, dispatch):
        return super().__new__(cls) if isinstance(dispatch, CDispatch) else None

    def __init__(self, dispatch):
        self._dispatch = dispatch
        for method_name in self._DISPATCH_METHODS:
            try:
                self._dispatch._FlagAsMethod(method_name)
            except pywintypes.com_error as e:
                if e.args[0] == -2147352570:
                    pass  # method_name doesn't exist in this version of wellcad or was not found, ignoring.")
                else:
                    raise

        for attribute_name in self._DISPATCH_ATTRIBUTES:
            try:
                entry = win32com.client.build.MapEntry(self._dispatch._oleobj_.GetIDsOfNames(0, attribute_name), (attribute_name,))
                self._dispatch._olerepr_.propMap[attribute_name] = entry
            except pywintypes.com_error as e:
                if e.args[0] == -2147352570:
                    pass  # attribute_name doesn't exist in this version of wellcad or was not found, ignoring.")
                else:
                    raise




from ._dispatch_wrapper import DispatchWrapper


class StackingPatternItem(DispatchWrapper):
    """Stores a stacking pattern over an interval.

    StackingPattern items are part of a StackingPattern Log and stores the top and bottom widths
    over a depth interval.
    Width values are between 0.0 and 1.0 and depth intervals can not overlap.

    Example
    -------
    >>> stack_log = borehole.log("Test")
    >>> stack_box = stack_box.insert_new_stack_item(10.5, 12.5, 0.5, 0.8)
    >>> stack_box.top_depth
    10.5
    >>> stack_box.top_depth
    12.5
    """

    @property
    def top_depth(self):
        """float: The top depth of the item in current depth
        units."""
        return self._dispatch.TopDepth

    @property
    def bottom_depth(self):
        """float: The bottom depth of the item in current depth
        units."""
        return self._dispatch.BottomDepth

    @property
    def top_width(self):
        """float: The top width of the item.
        
        This should be a value between 0.0 and 1.0. Setting it out of these
        bounds will clamp the value.
        """
        return self._dispatch.TopWidth

    @top_width.setter
    def top_width(self, value):
        self._dispatch.TopWidth = value

    @property
    def bottom_width(self):
        """float: The bottom width of the item.
        
        This should be a value between 0.0 and 1.0. Setting it out of these
        bounds will clamp the value.
        """
        return self._dispatch.BottomWidth

    @bottom_width.setter
    def bottom_width(self, value):
        self._dispatch.BottomWidth = value



from ._dispatch_wrapper import DispatchWrapper


class PolarAndRoseBox(DispatchWrapper):
    """A polar or rose diagram with annotations. These diagrams or "boxes" are stored in a Polar & Rose Log.

    Polar & Rose boxes are sometimes refered to as Schmit boxes (see example below)

    Example
    -------
    >>> log = borehole.log("Rose Diagram")
    >>> box = log.schmit_box_at_depth(10.5)  # Polar & Rose box called schmit box
    >>> box.text
    "Intervals Automatically Determined"
    """

    @property
    def top_depth(self):
        """float: The top depth of the Polar & Rose box in current
        depth reference units.
        """
        return self._dispatch.TopDepth

    @property
    def bottom_depth(self):
        """float: The bottom depth of the Polar & Rose box in current
        depth reference units.
        """
        return self._dispatch.BottomDepth

    @property
    def text(self):
        """str: The text for a Polar & Rose Log box.

        The text is not displayed in the plot and is only visible in
        the tabular editor or when exporting the data.
        """
        return self._dispatch.Text

    @text.setter
    def text(self, value):
        self._dispatch.Text = value





from ._dispatch_wrapper import DispatchWrapper
from ._font import Font
from ._drill_item import DrillItem
from ._structure import Structure
from ._litho_bed import LithoBed
from ._polar_and_rose_box import PolarAndRoseBox
from ._interval_item import IntervalItem
from ._fossil_item import FossilItem
from ._equipment_item import EquipmentItem
from ._comment_box import CommentBox
from ._marker_item import MarkerItem
from ._stacking_pattern_item import StackingPatternItem
from ._cross_section_box import CrossSectionBox
from ._litho_dictionary import LithoDictionary


class Log(DispatchWrapper):
    """The Log class represents a depth or time referenced set of data displayed as a column in a borehole document.
    Logs can be added/removed/manipulated from the borehole document itself.

    >>> import wellcad.com
    >>> app = wellcad.com.Application()
    >>> borehole = app.new_borehole()
    >>> log = borehole.insert_new_log(1) # Create a new well log
    """
    
    _DISPATCH_METHODS = ("Structure",)
    _DISPATCH_ATTRIBUTES = ("Style",)

    def file_export(self, directory, file_title=None, extension=None, prompt_user=None, config_filename=None):
        """Exports the data of the log in the specified format (TXT, CSV, ASC,
        WA* for all log types, BMP, TIF, GIF, JPG, PNG in addition for RGB and
        Image Logs).

        Parameters
        ----------
        directory : str
            Path to the location where the file should be stored.
        file_title : str, optional
            Name of the file that will be created. By default the log title
            will be taken.
        extension : str, optional
            The file extension to be used.
        prompt_user: bool, optional
            If set to ``True`` any warning messages during the graphic file
            export will be shown.
        config_filename : str, optional
            Configuration file used for ASCII (TXT, CSV, ASC, WA*) export only.
        
        Returns
        -------
        bool
            Whether the log was successfully exported.
        """
        return self._dispatch.FileExport(directory, file_title, extension, prompt_user, config_filename)

    @property
    def nb_of_data(self):
        """int: The number of data points in a log."""
        return self._dispatch.NbOfData

    @property
    def name(self):
        """str: The title of the log."""
        return self._dispatch.Name

    @name.setter
    def name(self, value):
        self._dispatch.Name = value

    @property
    def title_comment(self):
        """str: The title comment for this log."""
        return self._dispatch.TitleComment

    @title_comment.setter
    def title_comment(self, comment):
        self._dispatch.TitleComment = comment

    @property
    def top_depth(self):
        """float: The depth of the first top-most (shallowest) data point in
        the log using the current depth reference units."""
        return self._dispatch.TopDepth

    @property
    def bottom_depth(self):
        """float: The depth of the of the bottom-most (deepest) data point in
        the log using the current depth reference units."""
        return self._dispatch.BottomDepth


    @property
    def data_table(self):
        """tuple of tuples: The data table for a log. The first row in the data
        table is reserved for the log titles (e.g. for a Well Log the fist row
        in the data table contains the column titles "Depth" and the actual log
        title). The data format for each log equals the data displayed in the
        Tabular Editor."""
        return self._dispatch.DataTable

    @data_table.setter
    def data_table(self, data):
        self._dispatch.DataTable = data

    @property
    def data_min(self):
        """float: The minimum data value of the Well, Mud or Interval Log."""
        return self._dispatch.DataMin

    @property
    def data_max(self):
        """float: The maximum data value of the Well, Mud or Interval Log."""
        return self._dispatch.DataMax

    @property
    def log_unit(self):
        """str: The unit of a log. Restricted to log types having a unit in the log title."""
        return self._dispatch.LogUnit

    @log_unit.setter
    def log_unit(self, unit):
        self._dispatch.LogUnit = unit

    @property
    def left_position(self):
        """float: The position of the left side of the log column as a fraction
        of the document width.
        
        In the case that this is set to be a value higher than
        ``right_position``, the two attributes will swap. Values will be
        clamped in the range [0.0, 1.0].
        """
        return self._dispatch.LeftPosition

    @left_position.setter
    def left_position(self, position):
        self._dispatch.LeftPosition = position

    @property
    def right_position(self):
        """float: The position of the right side of the log column as a
        fraction of the document width.
        
        In the case that this is set to be a value lower than
        ``left_position``, the two attributes will swap. Values will be
        clamped in the range [0.0, 1.0].
        """
        return self._dispatch.RightPosition

    @right_position.setter
    def right_position(self, position):
        self._dispatch.RightPosition = position

    def set_position(self, left, right):
        """Sets the position and width of the log.

        Values given outside the range of [0.0, 1.0] will be clamped. If
        ``left`` > ``right``, the arguments are swapped.

        Parameters
        ----------
        left : float
            The position of the left side of the log column as a fraction of
            the document width.

        right : float
            The position of the right side of the log column as a fraction of
            the document width.
        """
        self._dispatch.SetPosition(left, right)

    @property
    def type(self):
        """int: The log type index.

        Log types are one of the below:

        * Undefined = 0
        * Well log = 1
        * Formula log = 2
        * Mud log = 3
        * FWS log = 4
        * Image log = 5
        * Structure log = 6
        * Litho log = 7
        * Comment log = 8
        * Engineering log = 9
        * RGB log = 10
        * Image Float 2 log = 11
        * Image float 4 log = 12
        * Interval log = 13
        * Analysis log = 14
        * Percentage log = 15
        * Coredesc log = 16
        * Depth log = 17
        * Strata log = 18
        * Stack log = 19
        * Polar & Rose log = 20
        * Cross log = 21
        * OLE log = 22
        * Shading log = 23
        * Marker log = 24
        * Breakout log = 25
        * Bio log = 26
        * Lineation log = 27
        """
        return self._dispatch.Type

    @property
    def hide_log_title(self):
        """bool: Whether the log title is hidden."""
        return self._dispatch.HideLogTitle

    @hide_log_title.setter
    def hide_log_title(self, value):
        self._dispatch.HideLogTitle = value

    @property
    def hide_log_data(self):
        """bool: Whether the log data is hidden."""
        return self._dispatch.HideLogData

    @hide_log_data.setter
    def hide_log_data(self, value):
        self._dispatch.HideLogData = value

    @property
    def log_background_color(self):
        """int: The background color of the log column.
        
        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values. Other allowed values are : 0xFFFFFFFF (None) and 0xFF000000 (automatic color).
        """
        return self._dispatch.LogBackgroundColor

    @log_background_color.setter
    def log_background_color(self, value):
        self._dispatch.LogBackgroundColor = value

    @property
    def border_style(self):
        """int: The border line style of the log column.
        
        Styles are specified as an integer:

        * Solid = 0
        * Dashed = 1
        * Dotted = 2
        * Dash-Dot = 3
        * Dash-dot-dot = 4
        """
        return self._dispatch.BorderStyle

    @border_style.setter
    def border_style(self, style):
        self._dispatch.BorderStyle = style

    @property
    def border_width(self):
        """float: The width for the log column border in 1/10 mm."""
        return self._dispatch.BorderWidth

    @border_width.setter
    def border_width(self, width):
        self._dispatch.BorderWidth = width

    @property
    def border_color(self):
        """int: The border color of the log column.
        
        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values. Other allowed values are : 0xFFFFFFFF (None) and 0xFF000000 (automatic color).
        """
        return self._dispatch.BorderColor

    @border_color.setter
    def border_color(self, value):
        self._dispatch.BorderColor = value

    @property
    def display_border(self):
        """bool: Whether the log column border is displayed."""
        return self._dispatch.DisplayBorder

    @display_border.setter
    def display_border(self, value):
        self._dispatch.DisplayBorder = value

    def clear_history(self):
        """Removes all entries from the log history."""
        self._dispatch.ClearHistory()

    @property
    def nb_of_history_item(self):
        """int: The number of entries in the history (audit trail) of a log."""
        return self._dispatch.NbOfHistoryItem

    def history_item_date(self, index):
        """Gets the date of the history item at the specified index.

        Parameters
        ----------
        index : int
            Zero based index of the history entry.
        
        Returns
        -------
        pywintypes.datetime
            The timezone-aware datetime of the history item.
        """
        return self._dispatch.HistoryItemDate(index).replace(tzinfo=None).astimezone()

    def history_item_description(self, index):
        """Gets the description of the history item at the specified index.

        Parameters
        ----------
        index : int
            Zero based index of the history entry.
        
        Returns
        -------
        str
            The description of the specified history entry.
        """
        return self._dispatch.HistoryItemDescription(index)

    @property
    def null_value(self):
        """float: The value that is treated as ``Null`` (not displayed) in a log."""
        return self._dispatch.NullValue

    @null_value.setter
    def null_value(self, value):
        self._dispatch.NullValue = value

    @property
    def mask_contacts(self):
        """bool: Whether contact lines within the log column are masked or not."""
        return self._dispatch.MaskContacts

    @mask_contacts.setter
    def mask_contacts(self, value):
        self._dispatch.MaskContacts = value

    @property
    def mask_horizontal_grid(self):
        """bool: Whether the horizontal (depth) gridlines of a log are masked or not."""
        return self._dispatch.MaskHorizontalGrid

    @mask_horizontal_grid.setter
    def mask_horizontal_grid(self, value):
        self._dispatch.MaskHorizontalGrid = value

    @property
    def sample_rate(self):
        """float: The sample interval of a log in current master depth units."""
        return self._dispatch.SampleRate

    @sample_rate.setter
    def sample_rate(self, rate):
        self._dispatch.SampleRate = rate

    @property
    def scale_low(self):
        """float: The low value of the log scale."""
        return self._dispatch.ScaleLow

    @scale_low.setter
    def scale_low(self, scale):
        self._dispatch.ScaleLow = scale

    @property
    def scale_high(self):
        """float: The high value of the log scale."""
        return self._dispatch.ScaleHigh

    @scale_high.setter
    def scale_high(self, scale):
        self._dispatch.ScaleHigh = scale

    @property
    def scale_mode(self):
        """int: The horizontal scale mode (linear or logarithmic) of a log.

        This property is only available for Well or Mud logs, and can have the
        following values:

        * Linear = 0
        * Logarithmic = 1
        """
        return self._dispatch.ScaleMode

    @scale_mode.setter
    def scale_mode(self, mode):
        self._dispatch.ScaleMode = mode

    @property
    def scale_reversed(self):
        """bool: Whether the data display scale is reversed."""
        return self._dispatch.ScaleReversed

    @scale_reversed.setter
    def scale_reversed(self, value):
        self._dispatch.ScaleReversed = value

    @property
    def use_log_colored_background(self):
        """bool: Whether the background color of a log is displayed."""
        return self._dispatch.UseLogColoredBackground

    @use_log_colored_background.setter
    def use_log_colored_background(self, value):
        self._dispatch.UseLogColoredBackground = value

    @property
    def maj_grid_enable(self):
        """bool: Whether to display major vertical gridlines."""
        return self._dispatch.MajGridEnable

    @maj_grid_enable.setter
    def maj_grid_enable(self, value):
        self._dispatch.MajGridEnable = value

    @property
    def min_grid_enable(self):
        """bool: Whether to display minor vertical gridlines."""
        return self._dispatch.MinGridEnable

    @min_grid_enable.setter
    def min_grid_enable(self, value):
        self._dispatch.MinGridEnable = value

    @property
    def maj_grid_spacing(self):
        """float: The spacing between major vertical gridlines."""
        return self._dispatch.MajGridSpacing

    @maj_grid_spacing.setter
    def maj_grid_spacing(self, spacing):
        self._dispatch.MajGridSpacing = spacing

    @property
    def min_grid_spacing(self):
        """float: The spacing between minor vertical gridlines."""
        return self._dispatch.MinGridSpacing

    @min_grid_spacing.setter
    def min_grid_spacing(self, spacing):
        self._dispatch.MinGridSpacing = spacing

    @property
    def lock_log_data(self):
        """bool: Whether the log data is protected from editing."""
        return self._dispatch.LockLogData

    @lock_log_data.setter
    def lock_log_data(self, value):
        self._dispatch.LockLogData = value

    def get_data(self, index):
        """Gets the data value for the specified index.

        This method is only applicable for a Well, Mud, Interval or Depth Log.

        Parameters
        ----------
        index : int
            Zero based index of the data to be retrieved.
        
        Returns
        -------
        float
            The value of the log data at the specified index.
        """
        return self._dispatch.GetData(index)

    def set_data(self, index, value):
        """Sets the data value for the specified index.

        This method is only applicable for a Well, Mud, Interval or Depth Log.

        Parameters
        ----------
        index : int
            Zero based index of the data to be retrieved.
        value : float
            The value you want to set the data to.
        """
        self._dispatch.SetData(index, value)

    def get_data_at_depth(self, depth):
        """Gets the log data value at the specified depth.
        
        This method is only applicable for a Well, Mud, Interval or Depth Log.

        Parameters
        ----------
        depth : float
            Depth value in current master depth units.
        
        Returns
        -------
        float
            The value of the log data at the specified depth.
        """
        return self._dispatch.GetDataAtDepth(depth)

    def set_data_at_depth(self, depth, value):
        """Sets the log data value at the specified depth.

        This method is only applicable for a Well, Mud, Interval or Depth Log.

        Parameters
        ----------
        depth : float
            Depth value in current master depth units.
        value : float
            The value you want to set the data to.
        """
        self._dispatch.SetDataAtDepth(depth, value)

    def data_depth(self, index):
        """Gets the log data depth for the specified index.
        
        This method can be called for Mud, Well, Depth, Percent, Analysis, FWS,
        Image and RGB Logs. For logs with a constant sample step (Well, Image,
        RGB, Analysis Logs), the index 0 corresponds to the Bottom Depth.

        Parameters
        ----------
        index : int
            Zero based index of the data to be retrieved.
        
        Returns
        -------
        float
            The depth of the log data at the specified index. If the index is
            out of bounds, this will be 0.0 for a Mud Log, and an extrapolated
            value for a Well Log.
        """
        return self._dispatch.DataDepth(index)

    def insert_data(self, index, value):
        """Inserts a new data value at the specified index.

        If necessary existing data points will be shifted.

        Parameters
        ----------
        index : int
            Zero based index at which the new data point will be inserted. The
            index must be lower or equal to the number of data points in the
            log.
        value : float
            The new data value.
        
        Raises
        ------
        pywintypes.com_error
            If the data couldn't be inserted (out-of-bounds index or other)
        """
        self._dispatch.InsertData(index, value)

    def insert_data_at_depth(self, depth, value):
        """Inserts a new data value at the specified depth.

        Data points above (shallower depth) will be shifted upward to
        accomodate the newly inserted point in a Well Log.

        Parameters
        ----------
        depth : float
            Depth in current master units at which the new data point should be
            inserted. Depth is rounded to the nearest sample for Well Logs.
        value : float
            The new data value
        """
        self._dispatch.InsertDataAtDepth(depth, value)

    @property
    def formula(self):
        """str: The mathematical formula used for a Formula log.
        
        Raises
        ------
        pywintypes.com_error
            If, during setting, the supplied formula is invalid.
        """
        return self._dispatch.Formula

    @formula.setter
    def formula(self, value):
        self._dispatch.Formula = value

    @property
    def filter(self):
        """int: The width (in samples) of the display filter used for Well Logs."""
        return self._dispatch.Filter

    @filter.setter
    def filter(self, value):
        self._dispatch.Filter = value

    @property
    def fixed_bar_width(self):
        """int: The fixed bar width, in units of 1/10 mm, for Mud Logs."""
        return self._dispatch.FixedBarWidth

    @fixed_bar_width.setter
    def fixed_bar_width(self, width):
        self._dispatch.FixedBarWidth = width

    def insert_new_interval_item(self, top_depth, bottom_depth, value):
        """Inserts a new interval in an Interval log.

        Parameters
        ----------
        top_depth : float
            The top depth of the new interval in current depth units.
        bottom_depth : float
            The bottom depth of the new interval in current depth units.
        value : float
            The value of the new interval item.
        
        Returns
        -------
        IntervalItem
            The newly inserted interval item.
        """
        return IntervalItem(self._dispatch.InsertNewIntervalItem(top_depth, bottom_depth, value))

    def interval_item(self, index):
        """Gets an interval item object from an Interval Log.

        Items are ordered by ascending top depth.

        Parameters
        ----------
        index : int
            Zero based index of the interval item.
        
        Returns
        -------
        IntervalItem or None
            The interval item at the specified index, or None if the index
            is out of range.
        """
        return IntervalItem(self._dispatch.IntervalItem(index))

    def interval_item_at_depth(self, depth):
        """Gets an interval item object from an Interval Log at the specified depth.

        Parameters
        ----------
        depth : float
            Depth value of the interval item, in current master depth units.
        
        Returns
        -------
        IntervalItem or None
            The interval item at the specified depth, or None if no interval
            item exists at the specified depth.
        """
        return IntervalItem(self._dispatch.IntervalItemAtDepth(depth))

    @property
    def pen_color(self):
        """int: The pen color for a Well or Mud log.
        
        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values.  Other allowed values are : 0xFFFFFFFF (None) and 0xFF000000 (automatic color).
        """
        return self._dispatch.PenColor

    @pen_color.setter
    def pen_color(self, color):
        self._dispatch.PenColor = color

    @property
    def pen_style(self):
        """int: The pen style for the Well or Mud log.
        
        Styles are specified as an integer:

        * Solid = 0
        * Dashed = 1
        * Dotted = 2
        * Dash-Dot = 3
        * Dash-dot-dot = 4
        """
        return self._dispatch.PenStyle

    @pen_style.setter
    def pen_style(self, style):
        self._dispatch.PenStyle = style

    @property
    def pen_width(self):
        """int: The pen width used in a Well or Mud Log in units of 1/10 mm."""
        return self._dispatch.PenWidth

    @pen_width.setter
    def pen_width(self, width):
        self._dispatch.PenWidth = width

    def remove_data(self, index):
        """Removes a data point from a Mud or Well Log.
        
        For Well Logs the data value will be set to ``Null``.

        Parameters
        ----------
        index : int
            Zero based index for the data point to be removed
        """
        self._dispatch.RemoveData(index)

    def remove_data_at_depth(self, depth):
        """Removes a data point from a Mud or Well Log.
        
        All data points above (shallower depth) the removed point will be
        shifted downward (deeper) when data is removed from a Well Log.

        Parameters
        ----------
        depth : float
            The depth value (in current master units) at which the data point.
            will be removed.
        """
        self._dispatch.RemoveDataAtDepth(depth)

    def remove_interval_item(self, index):
        """Removes a data interval from an Interval log.

        Parameters
        ----------
        index : int
            Zero based index to specify which data interval will be removed.
        """
        self._dispatch.RemoveIntervalItem(index)

    def remove_interval_item_at_depth(self, depth):
        """Removes a data interval from an Interval log.

        Parameters
        ----------
        depth : float
            Depth value in current master units to specify which interval item
            will be removed.
        """
        self._dispatch.RemoveIntervalItemAtDepth(depth)

    @property
    def shading(self):
        """int: The shading position used in a Well or Mud Log.
        
        * None = 0
        * Left = 1
        * Right = 2
        * Full = 3
        """
        return self._dispatch.Shading

    @shading.setter
    def shading(self, position):
        self._dispatch.Shading = position

    @property
    def style(self):
        """int: The data display style for a log.
        
        For Mud logs:

        * Fixed Bar = 1
        * Dynamic Bar = 2
        * Line = 3
        
        For Engineering logs:

        * Full = 0
        * Left = 1
        * Right = 2
        """
        return self._dispatch.Style

    @style.setter
    def style(self, style):
        self._dispatch.Style = style

    def attach_litho_dictionary(self, dictionary):
        """Attaches a new symbol or pattern library (\*.LTH file) to Litho, CoreDesc,
        Strata, Analysis or Percentage Log.

        Parameters
        ----------
        dictionary : str
            path and name of the LTH file to attach

        Returns
        -------
            LithoDictionary
                The LithoDictionary object
        """
        return LithoDictionary(self._dispatch.AttachLithoDictionary(dictionary))

    def get_component_name(self, column):
        """Gets the name (i.e. litho code) for the component used in
        the specified data column of a Percentage or Analysis Log.

        Parameters
        ----------
        column : int
            Zero based index of the data column in the tabular editor for
            which the component name should be set or retrieved.

        Returns
        -------
        str
            The code of the component used in the specified data
            column of a Percentage or Analysis Log.
        """
        return self._dispatch.GetComponentName(column)

    def set_component_name(self, column, code):
        """Sets the name (i.e. litho code) for the component used in
        the specified data column of a Percentage or Analysis Log.

        Parameters
        ----------
        column : int
            Zero based index of the data column in the tabular editor for
            which the component name should be set or retrieved.
        code : str
            Code of the component to be used in the specified data
            column of a Percentage or Analysis Log.
        """
        self._dispatch.SetComponentName(column, code)

    def fossil_item(self, index):
        """Gets a Fossil Item object from the CoreDesc Log at the specified index.

        Items are ordered by ascending top depth.

        Parameters
        ----------
        index : int
            Zero based index of the item to be retrieved.

        Returns
        -------
        FossilItem or None
            The FossilItem at the desired index or none if index is out of range.
        """
        return FossilItem(self._dispatch.FossilItem(index))

    def fossil_item_at_depth(self, depth):
        """Gets a fossil item object from the CoreDesc Log at the specified depth.

        Parameters
        ----------
        depth : float
            depth value in current depth units at which the item will be retrieved

        Returns
        -------
        FossilItem or None
            The FossilItem at the desired depth or None if depth is out of range.
        """
        return FossilItem(self._dispatch.FossilItemAtDepth(depth))

    def insert_new_fossil_item(self, top_depth, bottom_depth, litho_code, abundance, dominance, position):
        """Inserts a new data point or interval into a Core Description Log.

        Parameters
        ----------
        top_depth : float
            Top depth of the new data interval in current depth units.
        bottom_depth : float
            Bottom depth of the new data interval in current depth units.
        litho_code : str
            Code of the symbol representing the feature as defined in the symbol library of the log.
        abundance : int
            The abundance value associated with the symbol (e.g. between 0 and 9).
        dominance : int
            The dominance value associated with the symbol
            undiff = 0
            minor = 1
            major = 2
        position : float
            A value between 0 and 1 determining the horizontal position of the symbol within the log column.

        Returns
        -------
        FossilItem
            The newly created FossilItem.
        """
        return FossilItem(self._dispatch.InsertNewFossilItem(top_depth, bottom_depth, litho_code, abundance, dominance, position))

    def insert_new_litho_bed(self, top_depth, bottom_depth, litho_code, value, position):
        """Inserts a new lithology bed into a Litho Log.

        Parameters
        ----------
        top_depth : float
            Top depth of the new data interval in current depth units.
        bottom_depth : float
            Bottom depth of the new data interval in current depth units.
        litho_code : str
            Code of the symbol representing the feature as defined in the symbol library of the log.
        value : float
            Hardness value between 0 and 1.
        position : float
            A value between 0 and 1 determining the horizontal position of a non repeated symbol
            in percent of the track width.

        Returns
        -------
        LithoBed
            The newly created LithoBed.
        """
        return LithoBed(self._dispatch.InsertNewLithoBed(top_depth, bottom_depth, litho_code, value, position))

    def get_litho_bed(self, index):
        """Gets a LithoBed object at the specified index from a Lithology Log.

        Items are ordered by ascending top depth.

        Parameters
        ----------
        index : int
            Zero based index of the LithoBed to retrieve.

        Returns
        -------
        LithoBed
            The LithoBed at the desired index.
        """
        return LithoBed(self._dispatch.GetLithoBed(index))

    def set_litho_bed(self, index, litho_bed):
        """Sets a LithoBed object at the specified index from another
        LithoBed object.

        Parameters
        ----------
        index : int
            Zero based index of the LithoBed to retrieve.
        litho_bed : LithoBed
            The LithoBed object to copy.
        """
        self._dispatch.SetLithoBed(index, litho_bed._dispatch)

    def get_litho_bed_at_depth(self, depth):
        """Gets a LithoBed object at the specified depth from a Lithology Log.

        Parameters
        ----------
        depth : float
            Depth value in current depth units at which the item will be retrieved.

        Returns
        -------
        LithoBed
            The LithoBed at the desired depth.
        """
        return LithoBed(self._dispatch.GetLithoBedAtDepth(depth))

    def set_litho_bed_at_depth(self, depth, litho_bed):
        """Sets a LithoBed object at the specified depth from another
        LithoBed object.

        Parameters
        ----------
        depth : float
            Depth value in current depth units at which the item will be retrieved.
        litho_bed : LithoBed
            The LithoBed object to copy.
        """
        self._dispatch.SetLithoBedAtDepth(depth, litho_bed._dispatch)

    @property
    def litho_dictionary(self):
        """LithoDictionary: The symbol library used by the log as LithoDictionary object."""
        return LithoDictionary(self._dispatch.LithoDictionary)

    @litho_dictionary.setter
    def litho_dictionary(self, dictionary):
        self._dispatch.LithoDictionary = dictionary._dispatch

    def remove_fossil_item(self, index):
        """Removes an item at the specified index from a CoreDesc Log.

        Parameters
        ----------
        index : int
            Zero based index of the fossil item to be removed.
        """
        self._dispatch.RemoveFossilItem(index)

    def remove_fossil_item_at_depth(self, depth):
        """Removes an item at the specified index from a CoreDesc Log.

        Parameters
        ----------
        depth : float
            the depth value of the symbol in current depth units at which it will be removed.
        """
        self._dispatch.RemoveFossilItemAtDepth(depth)

    def remove_litho_bed(self, index):
        """Removes a lithology bed from the Lithology log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index of the lithology bed item to be removed.
        """
        self._dispatch.RemoveLithoBed(index)

    def remove_litho_bed_at_depth(self, depth):
        """Removes a lithology bed from the Lithology log at the specified depth.

        Parameters
        ----------
        depth : float
            the depth value in current depth units at which the lithological bed will be removed.
        """
        self._dispatch.RemoveLithoBedAtDepth(depth)

    def insert_trace(self, index):
        """Inserts a new data trace into an Image, FWS or Analysis Log at the specified index.

        Columns for the newly inserted trace are filled with No-Data
        value. Edit them with set_trace_data or set_trace_data_at_depth.

        Parameters
        ----------
        index : int
            Zero based index at which the trace should be added.
            Index zero represents the deepest trace.
            The maximum allowed index is (number of traces + 1).
            For Image and Analysis logs, existing traces will be shifted upwards depth wise.
        """
        self._dispatch.InsertTrace(index)

    def insert_trace_at_depth(self, depth):
        """Inserts a new data trace into an Image, FWS or Analysis or Percent Log at the specified depth.

        Columns for the newly inserted trace are filled with No-Data
        value. Edit them with set_trace_data or set_trace_data_at_depth.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the new
            data trace will be inserted.
            For Image and Analysis logs:
            Insertion depth is rounded to the nearest sample.
            Value must be within the depth range of the existing
            traces or contiguous to that range (± 1.5 * sampling rate).
            If necessary existing traces will be shifted upwards.
            For Percent and FWS logs, if there is already data at that
            depth, it is replaced with No-Data value, otherwise a trace
            is inserted at the desired depth without shifting other
            traces depth wise.

        """
        self._dispatch.InsertTraceAtDepth(depth)

    def remove_trace(self, index):
        """Remove an entire data trace from an Image, FWS, Analysis
        or Percentage Log.

        For Image logs, if the trace is the first or
        the last, it is removed. Otherwise, it sets all values of that
        trace to No-Data value.
        For Analysis, it is the same as for Image logs, but the
        replacement value is 0.
        For FWS logs, it removes the trace and shifts down all above
        traces.

        Parameters
        ----------
        index : int
            Zero based index of the trace (0 = bottom depth).
        """
        self._dispatch.RemoveTrace(index)

    def remove_trace_at_depth(self, depth):
        """Remove an entire data trace from an Image, FWS, Analysis or
        Percentage Log.

        For Image logs, if the trace is the first or
        the last, it is removed. Otherwise, it sets all values of that
        trace to No-Data value.
        For Analysis, it is the same as for Image logs, but the
        replacement value is 0.
        For FWS logs and Percent, it removes the trace and shifts down
        all above traces.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the trace
            will be removed or set to No-Data value.
        """
        self._dispatch.RemoveTraceAtDepth(depth)

    def get_trace_data(self, depth_index, trace_index):
        """Gets the data value at the specified row index and position within the trace
        (column index) of an Analysis, Percentage, FWS, Image or RGB Log.

        Parameters
        ----------
        depth_index : int
            zero based index of the depth (0 = bottom depth for FWS,
            Image, RGB and Analysis logs, top depth for Percent logs).
        trace_index : int
            zero based index of the column.

        Returns
        -------
        float
            The data value at the specified index and column.
            The current No-Data value (e.g. -999) will be returned if
            the depth or trace index refers to a non-existent (out of
            index range) data point.
        """
        return self._dispatch.GetTraceData(depth_index, trace_index)

    def set_trace_data(self, depth_index, trace_index, value):
        """Sets the data value at the specified row index and position within the trace
        (column index) of an Analysis, Percentage, FWS, Image or RGB Log.

        Parameters
        ----------
        depth_index : int
            zero based index of the depth (0 = bottom depth).
        trace_index : int
            zero based index of the column.
        value : float
            The value you want to set the data to.
        """
        self._dispatch.SetTraceData(depth_index, trace_index, value)

    def get_trace_data_at_depth(self, depth, trace_position):
        """Gets the data value at the specified depth and position within the trace of an Analysis,
        Percentage, FWS, Image or RGB Log.

        Parameters
        ----------
        depth : float
            The depth value at which you would like to retrieve the
            data value in the current depth units.
        trace_position : float
            The position within the trace (time or angle as shown in
            the column header of the tabular editor, not the index)
            at which you would like to retrieve the data value.

        Returns
        -------
        float
            The data value at the specified depth and column.
            The current No-Data value (e.g. -999) will be returned if
            the depth or trace index refers to a non-existent (out of
            index range) data point.
        """
        return self._dispatch.GetTraceDataAtDepth(depth, trace_position)

    def set_trace_data_at_depth(self, depth, trace_position, value):
        """set the data value at the specified depth and position within
        the trace of an Analysis, Percentage, FWS, Image or RGB Log

        Parameters
        ----------
        depth : float
            The depth value at which you would like to retrieve the data value in the current depth units.
        trace_position : float
            The position within the trace (time or angle as shown in the column header of the tabular editor,
            not the index) at which you would like to retrieve the data value.
        value : float
            The value you want to set the data to.
        """
        self._dispatch.SetTraceDataAtDepth(depth, trace_position, value)

    @property
    def trace_length(self):
        """int: The length of a data trace in Image, RGB and FWS Logs.

        For FWS, Analysis and Percent logs:
        If trace_length is set to a lower value than the current one,
        all the trace columns past the desired length are discared.
        If trace_length is set to a higher value than the current one,
        additional columns are filled with No-data values.
        For Image and RGB logs:
        If trace_length is set to a lower value than the current one,
        trace value are resampled and averaged.
        If trace_length is set to a higher value than the current one,
        TODO determine behaviour here.
        """
        return self._dispatch.TraceLength

    @trace_length.setter
    def trace_length(self, length):
        self._dispatch.TraceLength = length

    @property
    def trace_offset(self):
        """float: The offset of a data trace in the FWS Log."""
        return self._dispatch.TraceOffset

    @trace_offset.setter
    def trace_offset(self, offset):
        self._dispatch.TraceOffset = offset

    @property
    def trace_sample_rate(self):
        """float: The trace sample interval for a FWS Logs."""
        return self._dispatch.TraceSampleRate

    @trace_sample_rate.setter
    def trace_sample_rate(self, rate):
        self._dispatch.TraceSampleRate = rate

    def get_column_name(self, column):
        """Gets set the name of a Strata Log column.

        Parameters
        ----------
        column : int
            Zero based index of the column to be retrieved

        Returns
        -------
        str
            The name of the column.
        """
        return self._dispatch.GetColumnName(column)

    def set_column_name(self, column, name):
        """Sets set the name of a Strata Log column.

        Parameters
        ----------
        column : int
            Zero based index of the column to be retrieved
        name : str
            New name of the column.
        """
        self._dispatch.SetColumnName(column, name)

    def comment_box(self, index):
        """Gets the Comment Box object from the Comment Log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index at which the box will be retrieved.

        Returns
        -------
        CommentBox or None
            The CommentBox at the desired index or None if index is out of range.
        """
        return CommentBox(self._dispatch.CommentBox(index))

    def comment_box_at_depth(self, depth):
        """Gets the Comment Box object from the Comment Log at the specified depth.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the comment box will be retrieved.

        Returns
        -------
        CommentBox or None
            The CommentBox at the desired depth or None if depth is out of range.
        """
        return CommentBox(self._dispatch.CommentBoxAtDepth(depth))

    def insert_new_comment_box(self, top_depth, bottom_depth, text):
        """Sets a new box with the specified text into a Comment Log.

        Parameters
        ----------
        top_depth : float
            The top of the box in current depth units.
        bottom_depth : float
            The bottom of the box in current depth units.
        text : str
            The text to be displayed in the new box.

        Returns
        -------
        CommentBox
            The newly created CommentBox.
        """
        return CommentBox(self._dispatch.InsertNewCommentBox(top_depth, bottom_depth, text))

    def marker(self, index):
        """Gets the marker with the specified index.

        Parameters
        ----------
        index : int
            Zero based index of the marker.

        Returns
        -------
        MarkerItem
            The marker with the specified index.
        """
        return MarkerItem(self._dispatch.Marker(index))

    def marker_by_name(self, name):
        """Gets the marker with the specified name.

        Parameters
        ----------
        name : str
            Name of the marker to be retrieved.

        Returns
        -------
        MarkerItem
            The marker with the specified name.
        """
        return MarkerItem(self._dispatch.MarkerByName(name))

    def insert_new_marker(self, depth, name, comment, contact):
        """Inserts a new marker at the specified depth into a Marker Log

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the marker will be added.
        name : str
            The name (or identifier) of the marker.
        comment : str
            The optional comment for the marker.
        contact : str
            Name of a contact style to be used and available in the contact dictionary of the Marker Log.

        Returns
        -------
        MarkerItem
            The Marker object at the desired depth with the specified attributes.
        """
        return MarkerItem(self._dispatch.InsertNewMarker(depth, name, comment, contact))

    def remove_comment_box(self, index):
        """Removes a comment box from the Comment log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index at which the comment box will be removed.
        """
        self._dispatch.RemoveCommentBox(index)

    def remove_comment_box_at_depth(self, depth):
        """Removes a comment box from the Comment log at the specified depth.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the comment box will be removed.
        """
        self._dispatch.RemoveCommentBoxAtDepth(depth)

    def remove_marker(self, index):
        """Removes the marker from a Marker Log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index at which the marker box will be removed.
        """
        self._dispatch.RemoveMarker(index)

    def strata_column(self, index):
        """Gets a column from a Strata Log as Comment Log object.

        Parameters
        ----------
        index : int
            Zero based index  of the column to be returned.

        Returns
        -------
        Log
            A comment log object.
        """
        return Log(self._dispatch.StrataColumn(index))

    def remove_strata_column(self, index):
        """Removes the the specified column from a Strata Log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index of the column to be removed.
        """
        self._dispatch.RemoveStrataColumn(index)

    @property
    def font(self):
        """Gets the font used in a Comment Log as Font Object."""
        return Font(self._dispatch.Font)

    @font.setter
    def font(self, font):
        self._dispatch.Font = font._dispatch

    def attach_attribute_dictionary(self, attribute, file):
        """Attaches a new attribute library (\*.TAD file) to a
        Breakout, Lineation or Structure Log.

        Parameters
        ----------
        attribute : str
            Name of the classification column.
        file : str
            Path to the TAD file to attach.

        Raises
        ------
        pywintypes.com_error
            If the classification column doesn't exist, an exception is raised.
        """
        self._dispatch.AttachAttributeDictionary(attribute, file)

    def insert_new_attribute(self, attribute_name):
        """Inserts a new blank classification column to a Breakout,
        Lineation or Structure Log.

        Parameters
        ----------
        attribute_name : str
            Name of the new class.
        """
        self._dispatch.InsertNewAttribute(attribute_name)

    def get_attribute_name(self, index):
        """Gets the name of the attribute class (i.e. classification
        column) in a Breakout, Lineation or Structure Log.

        Parameters
        ----------
        index : str
            Zero based index of the column.

        Returns
        -------
        str
            The name of the attribute class (classification column)

        Raises
        ------
        pywintypes.com_error
            If the column index points to a non existent column (out
            of index range) an exception will be raised.
        """
        return self._dispatch.GetAttributeName(index)

    def set_attribute_name(self, index, name):
        """Sets the name of the attribute class (i.e. classification
        column) in a Breakout, Lineation or Structure Log.

        Parameters
        ----------
        index : str
            Zero based index of the column.
        name : str
            New name of the classification column.

        Raises
        ------
        pywintypes.com_error
            If the column index points to a non existent column (out
            of index range) an exception will be raised.
        """
        self._dispatch.SetAttributeName(index, name)

    def structure(self, index):
        """Gets a Structure object from the Structure Log at the
        specified index.

        Parameters
        ----------
        index : int
            Zero based index of the structure object to be retrieved.

        Returns
        -------
        Structure or None
            The structure at the specified index. If the index is outside the valid range no exception
            will be raised and ``None`` will be returned.
        """
        return Structure(self._dispatch.Structure(index))

    def structure_at_depth(self, depth):
        """Gets the closest Structure object from the Structure Log
        to the specified depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the
            structure object will be retrieved.

        Returns
        -------
        Structure or None
            The feature closest to the specified depth will be returned.
            If the depth is outside the valid range no exception
            will be raised and ``None`` will be returned.
        """
        return Structure(self._dispatch.StructureAtDepth(depth))

    def insert_new_structure_ex(self, depth, azimuth, dip, aperture):
        """Sets a new structure in a Structure Log.

        Parameters
        ----------
        depth : float
            The depth value in the current units of the breakout structure to
            be inserted.
        azimuth : float
            The azimuth angle of the structure measured in degrees.
        dip : float
            The tilt angle of the structure measured in degrees.
        aperture : float
            The aperture of the structure in meters.

        Returns
        -------
        Structure
            The newly created structure object.
        """
        return Structure(self._dispatch.InsertNewStructureEx(depth, azimuth, dip, aperture))

    def remove_structure(self, index):
        """Removes a structure from the Structure Log at the
        specified index.

        Parameters
        ----------
        index : int
            Zero based index of the structure to be removed.
        """
        self._dispatch.RemoveStructure(index)

    def remove_structure_at_depth(self, depth):
        """Removes a structure from the Structure Log at the
        specified depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the
            structure will be removed.
        """
        self._dispatch.RemoveStructureAtDepth(depth)

    def breakout(self, index):
        """Gets a breakout structure from the Breakout Log at the specified
        index.

        Parameters
        ----------
        index : int
            Zero based index of the breakout to be retrieved.

        Returns
        -------
        Structure or None
            The breakout at the specified index. If the index is
            outside the valid range no exception will be raised and
            ``None`` will be returned.

        """
        return Structure(self._dispatch.Breakout(index))

    def breakout_at_depth(self, depth):
        """Gets a breakout structure from the Breakout Log at the specified
        depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the
            breakout will be retrieved.

        Returns
        -------
        Structure or None
            The feature closest to the specified depth will be returned.
            If the depth is outside the valid range no exception
            will be raised and ``None`` will be returned.
        """
        return Structure(self._dispatch.BreakoutAtDepth(depth))

    def insert_new_breakout_ex(self, depth, azimuth, tilt, length, opening):
        """Sets a new breakout structure in a Breakout Log.

        If the mirror option is active, a second breakout structure object is
        added 180° degrees apart.

        Parameters
        ----------
        depth : float
            The depth value of the breakout structure in current depth units.
        azimuth : float
            The azimuth angle of the breakout structure measured in degrees.
        tilt : float
            The tilt angle of the breakout structure measured in degrees.
        length : float
            The length of the breakout structure in meters.
        opening : float
            The opening (or aperture) angle of the breakout structure in degrees.

        Returns
        -------
        Structure
            The newly created breakout structure.
        """
        return Structure(self._dispatch.InsertNewBreakoutEx(depth, azimuth, tilt, length, opening))

    def remove_breakout(self, index):
        """Removes a breakout structure from the Breakout Log at the specified
        index.

        Parameters
        ----------
        index : int
            Zero based index of the breakout structure to be removed.
        """
        self._dispatch.RemoveBreakout(index)

    def remove_breakout_at_depth(self, depth):
        """Removes a breakout structure from the Breakout Log at the specified
        depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the
            breakout structure will be removed.
        """
        self._dispatch.RemoveBreakoutAtDepth(depth)
    
    def lineation(self, index):
        """Gets a lineation pick from the Lineation Log at the specified
        index.

        Parameters
        ----------
        index : int
            Zero based index of the lineation to be retrieved.

        Returns
        -------
        Structure or None
            The lineation at the specified index. If the index is
            outside the valid range no exception will be raised and
            ``None`` will be returned.
        """
        return Structure(self._dispatch.Lineation(index))

    def lineation_at_depth(self, depth):
        """Gets a lineation structure from the Lineation Log at the specified
        depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the
            lineation will be retrieved.

        Returns
        -------
        Structure or None
            The feature closest to the specified depth will be returned.
            If the depth is outside the valid range no exception
            will be raised and ``None`` will be returned.
        """
        return Structure(self._dispatch.LineationAtDepth(depth))

    def insert_new_lineation_ex(self, depth, trend, plunge, eccentricity):
        """Sets a new linear structure in a Lineation Log.

        Parameters
        ----------
        depth : float
            Depth of the mid-point of the fitted line in current depth units.
        trend : float
            Trend direction of the vector in degrees.
        plunge : float
            "Dip" angle of the vector in degrees.
        eccentricity : float
            Offset of the lineation from the center of the borehole.
            A value between -1 and 1 can be set. An eccentricity of 0
            corresponds to a line going straight through the center
            of the borehole.

        Returns
        -------
        Structure
            The newly created lineation structure.
        """
        return Structure(self._dispatch.InsertNewLineationEx(depth, trend, plunge, eccentricity))

    def remove_lineation(self, index):
        """Removes a lineation from the Lineation Log at the
        specified index.

        Parameters
        ----------
        index : int
            Zero based index of the lineation to be removed.
        """
        self._dispatch.RemoveLineation(index)

    def remove_lineation_at_depth(self, depth):
        """Removes a lineation from the Lineation Log at the
        specified depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth value in current depth units at which the
            lineation will be removed.
        """
        self._dispatch.RemoveLineationAtDepth(depth)

    @property
    def length_unit(self):
        """float: The conversion factor (from meters) for the
        breakout structure length measured in the breakout log

        Set it to 0.001 when measured in mm and to 0.0254 when
        measured in inches."""
        return self._dispatch.LengthUnit

    @length_unit.setter
    def length_unit(self, factor):
        self._dispatch.LengthUnit = factor

    @property
    def caliper_unit(self):
        """float: The conversion factor (from meters) for the caliper
        value in a structure log.

        Set it to 0.001 when measured in mm and to 0.0254 when
        measured in inches."""
        return self._dispatch.CaliperUnit

    @caliper_unit.setter
    def caliper_unit(self, factor):
        self._dispatch.CaliperUnit = factor

    @property
    def aperture_unit(self):
        """float: The conversion factor (from meters) for the 
        aperture value in a structure log.  

        Set it to 0.001 when measured in mm and to 0.00254 when
        measured in 1/10th inches."""
        return self._dispatch.ApertureUnit

    @aperture_unit.setter
    def aperture_unit(self, factor):
        self._dispatch.ApertureUnit = factor

    def insert_new_schmit_box(self, top_depth, bottom_depth, text):
        """Inserts a new box into a Polar & Rose Log.

        Parameters
        ----------
        top_depth : float
            The top depth value of the interval in the current depth units.
        bottom_depth : float
            The bottom depth value of the interval in the current depth units.
        text : string
            A text description which is only shown in the tabular editor display.

        Returns
        -------
        PolarAndRoseBox
            The newly created Polar & Rose box.
        """
        return PolarAndRoseBox(self._dispatch.InsertNewSchmitBox(top_depth, bottom_depth, text))

    def schmit_box(self, index):
        """Gets a box object from the Polar & Rose Log at the
        specified index.

        Parameters
        ----------
        index : int
            Zero based index of the box to be returned.

        Returns
        -------
        PolarAndRoseBox
            The Polar & Rose box at the desired index.
        """
        return PolarAndRoseBox(self._dispatch.SchmitBox(index))

    def schmit_box_at_depth(self, depth):
        """Gets a box object from the Polar & Rose Log
        at the specified depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth of the box to be returned in current depth units.

        Returns
        -------
        PolarAndRoseBox
            The Polar & Rose box at the desired depth.
        """
        return PolarAndRoseBox(self._dispatch.SchmitBoxAtDepth(depth))

    def remove_schmit_box(self, index):
        """Removes a box from the Polar & Rose Log at the
        specified index.

        Parameters
        ----------
        index : int
            Zero based index of the Polar & Rose box to be removed.
        """
        self._dispatch.RemoveSchmitBox(index)

    def remove_schmit_box_at_depth(self, depth):
        """Removes a box from the Polar & Rose log at the specified
        depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth of the Polar & Rose box to be removed in
            current depth units.
        """
        self._dispatch.RemoveSchmitBoxAtDepth(depth)

    def cross_box(self, index):
        """Gets a Cross Box object from the Cross Section Log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index of the box to be retrieved.

        Returns
        -------
        CrossSectionBox or None
            The CrossSectionBox at the desired index or None if index is out of range.
        """
        return CrossSectionBox(self._dispatch.CrossBox(index))

    def cross_box_at_depth(self, depth):
        """Gets a Cross Box object from the Cross Section Log at the specified depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth of the box to be retrieved in current depth units.

        Returns
        -------
        CrossSectionBox or None
            The CrossSectionBox at the desired index or None if the index is out of range.
        """
        return CrossSectionBox(self._dispatch.CrossBoxAtDepth(depth))

    def insert_new_cross_box(self, top_depth, bottom_depth):
        """Inserts a new box into the Cross Section Log.

        Parameters
        ----------
        top_depth : float
            The top depth value of the cross section box in current depth units.
        bottom_depth : float
            The bottom depth value of the cross section box in current depth units.

        Returns
        -------
        CrossSectionBox
            The newly created CrossSectionBox.
        """
        return CrossSectionBox(self._dispatch.InsertNewCrossBox(top_depth, bottom_depth))

    def remove_cross_box(self, index):
        """Removes a box from the Cross Section Log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index of the box to be removed.
        """
        self._dispatch.RemoveCrossBox(index)

    def remove_cross_box_at_depth(self, depth):
        """Removes a box from the Cross Section Log at the specified depth.

        Parameters
        ----------
        depth : float
            The depth in current units at which the box will be removed.
        """
        self._dispatch.RemoveCrossBoxAtDepth(depth)

    def insert_new_stack_item(self, top_depth, bottom_depth, top_width, bottom_width):
        """Inserts a new data interval into a Stacking Pattern Log.

        Parameters
        ----------
        top_depth : float
            The top depth value of the interval in the current depth units.
        bottom_depth : float
            The bottom depth value of the interval in the current depth units.
        top_width : float
           Width value at the top of the new interval.
        bottom_width : float
            Width value at the bottom of the new interval.

        Returns
        -------
        StackingPatternItem
            The newly created StackingPatternItem.
        """
        return StackingPatternItem(self._dispatch.InsertNewStackItem(top_depth, bottom_depth, top_width, bottom_width))

    def stack_item(self, index):
        """Gets a Stack Item object from the Stacking Pattern Log at the specified depth index.

        Items are ordered by ascending top depth.

        Parameters
        ----------
        index : int
            Zero based index of the item to be retrieved.

        Returns
        -------
        StackingPatternItem or None
            The StackingPatternItem at the desired index or None if the index is out of range.
        """
        return StackingPatternItem(self._dispatch.StackItem(index))

    def stack_item_at_depth(self, depth):
        """Gets a Stack Item object from the Stacking Pattern Log at the specified depth in current depth units.

        Parameters
        ----------
        depth : float
            The depth of the item to be retrieved in current depth units.

        Returns
        -------
        StackingPatternItem or None
            The StackingPatternItem at the desired depth or None if depth is out of range.
        """
        return StackingPatternItem(self._dispatch.StackItemAtDepth(depth))

    def remove_stack_item(self, index):
        """Removes an item from the Stacking Pattern Log at the specified index.

        Parameters
        ----------
        index : int
            Zero based index  of the stacking pattern box to be removed.
        """
        self._dispatch.RemoveStackItem(index)

    def remove_stack_item_at_depth(self, depth):
        """Removes an item from the Stacking Pattern Log at the specified depth.

        Parameters
        ----------
        depth : float
            The depth of the stack item to be removed in current depth units.
        """
        self._dispatch.RemoveStackItemAtDepth(depth)

    @property
    def used_as_depth_scale(self):
        """bool: Whether the Depth Log is used as the current depth
        reference axis."""
        return self._dispatch.UsedAsDepthScale

    @used_as_depth_scale.setter
    def used_as_depth_scale(self, mode):
        self._dispatch.UsedAsDepthScale = mode

    def insert_new_ole_box_from_file(self, file_name, allow_picture, top_depth, bottom_depth):
        """Inserts a new OLE object between the specified interval
        into an OLE Log.

        OLE objects can be files such as images, Excel charts,
        Word documents or pdfs.

        Parameters
        ----------
        file_name : str
            Path and name of the file to be loaded.
        allow_picture : bool
            Set to True to allow graphic files to be displayed using
            an internal viewer.
        top_depth : float
            Top depth of the OLE object in current depth units.
        bottom_depth : float
            Bottom depth of the OLE object in current depth units.
        """
        self._dispatch.InsertNewOleBoxFromFile(file_name, allow_picture, top_depth, bottom_depth)

    def drill_item(self, index):
        """Gets a Drill Item object from the Engineering Log at the
        specified index.

        Parameters
        ----------
        index : int
            Zero based index at which the drill item will be retrieved.

        Returns
        -------
        DrillItem
            The drill item at the specified index.
        """
        return DrillItem(self._dispatch.DrillItem(index))

    def drill_item_at_depth(self, depth):
        """Gets a Drill Item object from the Engineering Log at the
        specified depth.

        Parameters
        ----------
        depth : float
            The depth of the item to be retrieved in current depth
            units.

        Returns
        -------
        DrillItem
            The drill item at the specified depth.
        """
        return DrillItem(self._dispatch.DrillItemAtDepth(depth))

    def eqp_item(self, index):
        """Gets an Equipment Item object at the specified index from
        the Engineering Log.

        Parameters
        ----------
        index : int
            Zero based index at which the item will be retrieved.

        Returns
        -------
        EquipmentItem
            The equipment item at the specified index.
        """
        return EquipmentItem(self._dispatch.EqpItem(index))

    @property
    def comment_style(self):
        """int: The position of the comment associated with an engineering log.

        The following styles are available:

            * 0 = None, comment is not displayed
            * 1 = Left, comment displayed on the left of the engineering log
            * 2 = Right, comment displayed on the right of the engineering log
        """
        return self._dispatch.CommentStyle

    @comment_style.setter
    def comment_style(self, style):
        self._dispatch.CommentStyle = style

    @property
    def ground_depth(self):
        """float: The starting point (reference datum) of the
        borehole."""
        return self._dispatch.GroundDepth

    @ground_depth.setter
    def ground_depth(self, starting_point):
        self._dispatch.GroundDepth = starting_point

    @property
    def diameter_high(self):
        """float: The maximum diameter scaling value
        (width of the log column) for an Engineering Log."""
        return self._dispatch.DiameterHigh

    @diameter_high.setter
    def diameter_high(self, diameter):
        self._dispatch.DiameterHigh = diameter

    def insert_new_drill_item(self, bottom_depth, diameter):
        """Inserts a new drill item into the Engineering Log.

        Item index are ordered by depth value in the stack of drill
        items.

        Parameters
        ----------
        bottom_depth : float
            The bottom depth of the borehole in current depth units.
            (The top depth is either the ground_depth or the former
            bottom depth).
        diameter : float
            The diameter of the borehole.

        Returns
        -------
        DrillItem
            The inserted DrillItem
        """
        return DrillItem(self._dispatch.InsertNewDrillItem(bottom_depth, diameter))

    def insert_new_eqp_item(self, top_depth, bottom_depth, name):
        """Inserts a new equipment item of the specified name and
        depth interval into the Engineering Log.

        The new item is given the last index in the list of equipment
        items.

        Parameters
        ----------
        top_depth : float
            The top depth of the equipment item interval
            in current units.
        bottom_depth : float
            The bottom depth of the equipment item interval
            in current units.
        name : str
            The name (code) of the equipment item to be inserted.
            Base available equipments names are listed below, but you
            can modify or add your own in the Equipment dictionary.

                * PlainCasing
                * WireWoundCasing
                * SlottedCasing
                * PerforatedCasing
                * Centralizer
                * Shoe
                * Packer
                * Water
                * Wedge
                * HeadWorks
                * Transducer
                * Gauge
                * Cement
                * Gravel
                * NormalThread
                * ReverseThread
                * Plug

            If an invalid name is provided, no item will be inserted.

        Returns
        -------
        EqpItem
            The inserted EqpItem
        """
        return EquipmentItem(self._dispatch.InsertNewEqpItem(top_depth, bottom_depth, name))

    @property
    def nb_of_drill_item(self):
        """int: The number of drill items in an Engineering Log."""
        return self._dispatch.NbOfDrillItem

    @property
    def nb_of_eqp_item(self):
        """int: The number of equipment items in an Engineering Log."""
        return self._dispatch.NbOfEqpItem

    def remove_drill_item(self, index):
        """Removes the drill item at the specified index from an
        Engineering Log.

        Parameters
        ----------
        index : int
            Zero based index at which the item will be removed.
        """
        self._dispatch.RemoveDrillItem(index)

    def remove_eqp_item(self, index):
        """Removes an equipment item at the specified depth index
        from an Engineering Log.

        Parameters
        ----------
        index : int
            Zero based index at which the item will be removed.
        """
        self._dispatch.RemoveEqpItem(index)

    @property
    def background_color(self):
        """int: The background color of the engineering log.

        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values.
        """
        return self._dispatch.BackgroundColor

    @background_color.setter
    def background_color(self, value):
        self._dispatch.BackgroundColor = value


    @property
    def background_hatch_style(self):
        """int: The background hatch style for the Engineering Log.

        Available styles are:

        * 0: horizontal
        * 1: vertical
        * 2: forward diagonal
        * 3: backward diagonal
        * 4: cross
        * 5: diagonal cross

        If an invalid style is set, nothing will happen.
        """
        return self._dispatch.BackgroundHatchStyle

    @background_hatch_style.setter
    def background_hatch_style(self, code):
        self._dispatch.BackgroundHatchStyle = code

    @property
    def background_style(self):
        """int: The background style for the Engineering Log.

        Available styles are:

        * 0: none
        * 1: solid
        * 2: hatch

        If an invalid style is set, nothing will happen.
        """
        return self._dispatch.BackgroundStyle

    @background_style.setter
    def background_style(self, code):
        self._dispatch.BackgroundStyle = code

    def allow_export_attribute_dictionary(self, index, export, password):
        """When dealing with a protected document you can use this method to enable/disable the option
        to export the tadpole dictionary (\*.TAD) from a specific classification column of a Structure or
        Breakout Log. This assumes you are in possession of the password.

        Parameters
        ----------
        index : int
            Zero based index of the classification column for which the protection level should eb changed.
        export : bool
            Set this boolean to True to allow an export as \*.tad file. Set it to False to protect the dictionary.
        password : str
            the password needed to make changes to the protection level.
        """
        self._dispatch.AllowExportAttributeDictionary(index, export, password)

    def allow_export_litho_dictionary(self, export, password):
        """When dealing with a protected document you can use this method to enable / disable the option
        to export the symbol dictionary (\*.LTH) of a log. This assumes you are in possession of the password.

        Parameters
        ----------
        export : bool
            Set this boolean to True to allow the dictionary export. Set it to False to protect the dictionary.
        password : str
            the password needed to make changes to the protection level.
        """
        self._dispatch.AllowExportLithoDictionary(export, password)

    def allow_modify_log_data(self, export, password):
        """When dealing with a protected document you can use this method to enable / disable the option
        to edit the data of a log. This assumes you are in possession of the password.

        Parameters
        ----------
        export : bool
            Set this boolean to True to allow the modification of log data. Set it to False to protect the data.
        password : str
            the password needed to make changes to the protection level.
        """
        self._dispatch.AllowModifyLogData(export, password)

    def allow_modify_log_settings(self, export, password):
        """When dealing with a protected document you can use this method to enable / disable the option
        to change the settings of a log. This assumes you are in possession of the password.

        Parameters
        ----------
        export : bool
            Set this boolean to True to allow access to the log settings. Set it to False to protect the log settings.
        password : str
            the password needed to make changes to the protection level.
        """
        self._dispatch.AllowModifyLogSettings(export, password)

    def allow_use_formula(self, export, password):
        """When dealing with a protected document you can use this method to enable / disable the option
        to access and use the formula in a Formula Log. This assumes you are in possession of the password.

        Parameters
        ----------
        export : bool
            Set this boolean to True to allow access the formula. Set it to False to protect the formula.
        password : str
            the password needed to make changes to the protection level.
        """
        self._dispatch.AllowUseFormula(export, password)

    def allow_view_formula(self, export, password):
        """When dealing with a protected document you can use this method to enable / disable the option
        to see the formula used in a Formula Log. This assumes you are in possession of the password.

        Parameters
        ----------
        export : bool
            Set this boolean to True to see the formula. Set it to False to protect the formula.
        password : str
            the password needed to make changes to the protection level.
        """
        self._dispatch.AllowViewFormula(export, password)

    def allow_view_log_history(self, export, password):
        """When dealing with a protected document you can use this method to enable / disable the option
        to view the log history. This assumes you are in possession of the password.

        Parameters
        ----------
        export : bool
            Set this boolean to True to access the log history. Set it to False to protect the log history.
        password : str
            the password needed to make changes to the protection level.
        """
        self._dispatch.AllowViewLogHistory(export, password)



from ._dispatch_wrapper import DispatchWrapper


class Font(DispatchWrapper):
    """ This class encapsulates the properties of fonts used throughout WellCAD

    For example, fonts can be specified in the following places:

    * Comment log
    * Comment partition
    * Property partition
    * Log title

    In general, attributes of this class reflect those of the ``LOGFONT``
    structure in the Win32 API. Documentation for this structure can be found
    `here <https://docs.microsoft.com/en-us/windows/win32/api/wingdi/ns-wingdi-logfonta>`_.

    Example
    -------
    >>> log = borehole.log("Comments")
    >>> font = log.font
    >>> font.name
    'Arial Narrow'
    >>> font.italic
    False
    """

    @property
    def name(self):
        """str: The name of the font type used."""
        return self._dispatch.Name

    @name.setter
    def name(self, new_name):
        self._dispatch.Name = new_name

    @property
    def weight(self):
        """int: The weight (boldness) of the font used.

        Values typically range from 100 to 900 where
        400 is regular weight and 700 is bold.
        """
        return self._dispatch.Weight

    @weight.setter
    def weight(self, new_weight):
        self._dispatch.Weight = new_weight

    @property
    def italic(self):
        """bool: Whether the font is italicized."""
        return self._dispatch.Italic

    @italic.setter
    def italic(self, flag):
        self._dispatch.Italic = flag

    @property
    def underline(self):
        """bool: Whether the font is underlined."""
        return self._dispatch.Underline

    @underline.setter
    def underline(self, flag):
        self._dispatch.Underline = flag

    @property
    def bold(self):
        """bool: Whether the font is bold."""
        return self._dispatch.Bold

    @bold.setter
    def bold(self, flag):
        self._dispatch.Bold = flag

    @property
    def strikethrough (self):
        """bool: Whether the font is struck through."""
        return self._dispatch.Strikethrough

    @strikethrough.setter
    def strikethrough(self, flag):
        self._dispatch.Strikethrough = flag

    @property
    def size(self):
        """int: The size of the font.

        See the Win32 ``LOGFONT`` documentation for more of an
        explanation.
        """
        return self._dispatch.Size

    @size.setter
    def size(self, new_size):
        self._dispatch.Size = new_size

    @property
    def charset(self):
        """int: The index of the character set used.
        
        The character set can be selected from a list of available values
        documented in the Win32 ``LOGFONT`` documentation).
        """
        return self._dispatch.Charset

    @charset.setter
    def charset(self, new_charset):
        self._dispatch.Charset = new_charset



from ._dispatch_wrapper import DispatchWrapper


class FossilItem(DispatchWrapper):
    """ This class represents a fossil item. These are items within a CoreDesc log

    You can find more information on your defined fossils in your LithCAD library.

    Example
    -------
    >>> log = borehole.log("Fossils")
    >>> item = log.fossil_item_at_depth(10.5)
    >>> item.symbol_code
    'undulating'
    >>> item.abundance = 2
    """

    @property
    def top_depth(self):
        """float: The top depth of the item in current depth
        units."""
        return self._dispatch.TopDepth

    @property
    def bottom_depth(self):
        """float: The bottom depth of the item in current depth
        units."""
        return self._dispatch.BottomDepth

    @property
    def symbol_code(self):
        """str: The code of the item."""
        return self._dispatch.SymbolCode

    @symbol_code.setter
    def symbol_code(self, value):
        self._dispatch.SymbolCode = value

    @property
    def abundance(self):
        """float: The abundance of the item.

        Allowed values are :
        * 1 = rare
        * 2 = moderate
        * 3 = common
        * 4 = abundant
        * 5 = pervasive
        """
        return self._dispatch.Abundance

    @abundance.setter
    def abundance(self, value):
        self._dispatch.Abundance = value

    @property
    def dominance(self):
        """float: The dominance of the item.

        Allowed values are :

        * 0 = undifferentiated
        * 1 = minor
        * 2 = major
        """
        return self._dispatch.Dominance

    @dominance.setter
    def dominance(self, value):
        self._dispatch.Dominance = value



from win32com.client import Dispatch
from ._dispatch_wrapper import DispatchWrapper
from ._borehole import Borehole

class Application(DispatchWrapper):
    """The core class used to interact with WellCAD via its COM API.

    In general, it is the only class you should instantiate directly. All
    interaction with borehole documents, logs, etc, should be done through
    this initial ``Application`` instance.

    Instantiating an ``Application`` will automatically open a WellCAD instance
    if there is not one already open, or connect to an existing WellCAD
    instance if there is one open.

    Before using the COM API, ensure WellCAD has been started at least once by
    an administrator in order to register it in the computer registry.

    Example
    -------
    >>> import wellcad.com
    >>> app = wellcad.com.Application()
    >>> app.show_window()
    True
    >>> app.new_borehole()
    <wellcad.com._borehole.Borehole object at 0x000001D6973FBD30>
    """

    _DISPATCH_METHODS = ("ShowWindow", "NewBorehole", "OpenBorehole",
        "GetBorehole", "GetActiveBorehole", "FileImport", "MultiFileImport")
    
    def __new__(cls):
        return object.__new__(cls)
    
    def __init__(self):
        super().__init__(Dispatch("WellCAD.Application"))
        
    def show_window(self):
        """Attempts to display the WellCAD workspace on screen.
        
        Returns
        -------
        bool
            Whether the window was successfully shown.
        """
        return self._dispatch.ShowWindow()
        
    def minimize_window(self):
        """Collapses the application window to an icon."""
        self._dispatch.MinimizeWindow()
    
    def maximize_window(self):
        """Extends the application window to full screen."""
        self._dispatch.MaximizeWindow()
    
    def cascade(self):
        """Cascades all borehole document windows (unless tabbed)."""
        self._dispatch.Cascade()
    
    def tile_horizontally(self):
        """Arranges document windows horizontally (unless tabbed)."""
        self._dispatch.TileHorizontally()
    
    def tile_vertically(self):
        """Arranges document windows vertically (unless tabbed)."""
        self._dispatch.TileVertically()
    
    def new_borehole(self, template=None):
        """Creates a new borehole document object.

        The new borehole object is blank unless a template file is provided.
        
        Parameters
        ----------
        template : str, optional
            An optional path to a borehole document template (.wdt).
            If provided, the new borehole will be created using the
            template.
        
        Returns
        -------
        Borehole
            The new borehole document.
        """
        return Borehole(self._dispatch.NewBorehole(template))

        
    def open_borehole(self, path=None):
        """Opens a WellCAD borehole document file (.wcl file).
        
        Parameters
        ----------
        path : str, optional
            The file path to the WellCAD borehole document file to open. If no
            file path is provided, the user will be prompted to select a file
            using a standard File Open dialog box.
        
        Returns
        -------
        Borehole or None
            The opened borehole document, or ``None`` if opening the file
            failed.
        """
        return Borehole(self._dispatch.OpenBorehole(path))
        

    def get_borehole(self, index=None):
        """Gets an existing borehole document by index.
        
        Gets a borehole document object according to the zero-based index of
        the document open in WellCAD.
        
        Parameters
        ----------
        index : int, optional
            A zero based index of the borehole document. If no index is
            provided, the index of the most recently used document is used.
        
        Returns
        -------
        Borehole or None
            The borehole document object with the specified index, or ``None``
            if the index is out of bounds.
        """
        return Borehole(self._dispatch.GetBorehole(index))
        
        
    def get_active_borehole(self):
        """Gets the active borehole document (i.e. the one that currently has focus).
                   
        Returns
        -------
        Borehole or None
            The current active borehole document object, or ``None`` if no
            document has focus.
        """
        return Borehole(self._dispatch.GetActiveBorehole())
        
        
    def close_borehole(self, prompt_for_saving=None, index=None):
        """Closes a specific borehole document.
        
        The borehole document that is closed can be specified by an index,
        or, if none is provided, the most recently opened borehole document
        will be closed.
        
        Parameters
        ----------
        prompt_for_saving : bool, optional
            Whether or not to prompt the user to save the borehole document.
        index : int or str, optional
            The (zero-based) index of the document to close, or the document
            name. If not provided, the most recently opened document will be
            closed.
        """
        self._dispatch.CloseBorehole(prompt_for_saving, index)
    
    @property
    def nb_of_documents(self):
        """int: The number of borehole documents open in WellCAD."""
        return self._dispatch.NbOfDocuments
    
    def file_import(self,
                    file_name=None,
                    prompt_user=None,
                    config_file=None,
                    log_file=None):
        """Imports the specified file into a new borehole document.
        
        Allows import of TFD, LAS, DLIS, TXT, CSV and other file
        formats into WellCAD. Please refer to the WellCAD help
        file for a description of all import parameters to be used
        in the configuration file / parameter string. If filename
        is left blank the File Open dialog will be displayed. 

        Parameters
        ----------
        file_name : str, optional
            The path of the file to import.
        prompt_user : bool, optional
            Whether to display an import dialog box to allow the user
            to specify import settings.
        config_file : str, optional
            Path and filename of the configuration file or parameter string.
        log_file : str, optional
            Path and name of the file to log error messages.
        
        Returns
        -------
        Borehole
            A new borehole document containing the imported data.
        """
        return Borehole(self._dispatch.FileImport(file_name,
                                                  prompt_user,
                                                  config_file,
                                                  log_file))
        
    def multi_file_import(self,
                          file_name=None,
                          prompt_user=None,
                          config_file=None,
                          log_file=None):
        """Creates a single borehole document from all specified files.

        Allows import of multiple TFD, LAS, DLIS, TXT, CSV and other
        file formats into WellCAD. Imported data will be merged into
        the same document. Please refer to the WellCAD help file for
        a description of all import parameters to be used in the
        configuration file / parameter string. If filename is left
        blank the File Open dialog will be displayed.

        Parameters
        ----------
        file_name : str, optional
            A comma separated list of input files (path and file name). 
        prompt_user : bool, optional
            Boolean to display the import dialog boxes.
        config_file : str, optional
            Path and filename of the configuration file or parameter string.
        log_file : str, optional
            Path and name of the file to log error messages.
        
        Returns
        -------
        Borehole
            A newly created borehole document containing the data from the
            imported files.
        """
        return Borehole(self._dispatch.MultiFileImport(file_name,
                                                       prompt_user,
                                                       config_file,
                                                       log_file))       
    
    def quit(self, prompt_for_saving=None):
        """Closes all borehole documents and exits WellCAD.
        
        Parameters
        ----------
        prompt_for_saving : bool, optional
            Whether to display the Save As dialog box for unsaved documents.
            Default behaviour is to prompt the user.
        """
        self._dispatch.Quit(prompt_for_saving)


from ._dispatch_wrapper import DispatchWrapper


class Header(DispatchWrapper):
    """The class that manages the headers fields.

    Example
    -------
    >>> header = borehole.header
    >>> for i in range(header.nb_of_items):
    >>>     print(header.item_name(i))
    COMPANY
    Title
    STATE
    FIELD
    Features
    email
    Complementary1
    Complementary2
    Tools
    """

    @property
    def nb_of_items(self):
        """int: Number of dynamic text fields in the entire header."""
        return self._dispatch.NbOfItems

    def get_item_text(self, name):
        """Contents of a dynamic text field in the header.

        Parameters
        ----------
        name : str
            ID of the text field as entered in HeadCAD.

        Returns
        -------
        str
            The text contained in the dynamic text field.
        """

        return self._dispatch.GetItemText(name)

    def set_item_text(self, name, text):
        """Adds the content of a dynamic text field in the header.

        Parameters
        ----------
        name : str
            ID of the text field as entered in HeadCAD.
        text : str
            Text to add into the dynamic text field.
        """

        self._dispatch.SetItemText(name, text)

    def item_name(self, index):
        """Returns the ID of a dynamic text field.

        Returns the ID of a dynamic text field in the header based on
        the zero based index. Use GetNbOfItems to retrieve the total
        number of text fields in the header.

        Parameters
        ----------
        index : int
            Zero based index of the dynamic text field.

        Returns
        -------
        str
            The ID of the dynamic text field.
        """

        return self._dispatch.ItemName(index)

    def allow_export_header(self, index, enable, password):
        """Changes the protection status to export the header design

        When dealing with a protected document you can use this method to
        enable / disable the protection to export the header attached to a
        borehole document as a WCH file. This assumes you are in possession
        of the password. You still have to export the header manually,
        this method only toggles the protection.

        Parameters
        ----------
        index : int
            index of the sub-header
        enable : bool
            Set this boolean to True to allow the export of the header.
            Set it to False to protect the header again.
        password : str
            String of the password needed to make changes to the
            protection level of a header.
        """

        self._dispatch.AllowExportHeader(index, enable, password)

    def allow_export_trailer(self, index, enable, password):
        """Changes the protection status to export the trailer design

        When dealing with a protected document you can use this method to
        enable / disable the protection to export the trailer attached to a
        borehole document as a WCH file. This assumes you are in possession
        of the password. You still have to export the trailer manually,
        this method only toggles the protection.

        Parameters
        ----------
        index : int
            index of the sub-trailer
        enable : bool
            Set this boolean to True to allow the export of the trailer.
            Set it to False to protect the trailer again.
        password : str
            String of the password needed to make changes to the
            protection level of a trailer.
        """

        self._dispatch.AllowExportTrailer(index, enable, password)



from ._dispatch_wrapper import DispatchWrapper


class CommentBox(DispatchWrapper):
    @property
    def top_depth(self):
        """float: The top depth of the comment box in current depth
        units."""
        return self._dispatch.TopDepth

    @property
    def bottom_depth(self):
        """float: The bottom depth of the comment box in current depth
        units."""
        return self._dispatch.BottomDepth

    @property
    def text(self):
        """str: The text of the comment box."""
        return self._dispatch.Text

    @text.setter
    def text(self, value):
        self._dispatch.Text = value




from ._dispatch_wrapper import DispatchWrapper

class Odbc(DispatchWrapper):
    """ An ODBC object allows interaction with databases from within WellCAD.

    Example
    -------
    >>> odbc = borehole.odbc
    >>> odbc.interpret_sql_statement('$dsn = OpenDatabase(Test, Admin)')
    >>> odbc.interpret_sql_statement('$log = SELECT Depth, Value FROM tblGamma')
    """

    def interpret_sql_statement(self, statement):
        """Executes the SQL statement provided

        A user manual documenting the syntax of the SQL statements that can be
        used can be obtained by contacting support@alt.lu.

        Parameters
        ----------
        statement : str
            The SQL statement to execute.

        Returns
        -------
        bool
            Whether the statement executed successfully.
        """
        return self._dispatch.InterpretSQLStatement(statement)



from ._dispatch_wrapper import DispatchWrapper
from ._litho_pattern import LithoPattern


class LithoDictionary(DispatchWrapper):
    """A dictionary containing patterns used to represent lithologies.

    You can find more information on your defined lithologies in your LithCAD library.

    Example
    -------
    >>> log = borehole.log("Lithology")
    >>> dict = log.litho_dictionary
    >>> dict.name
    'Sst'
    >>> dict.nb_of_patterns
    12
    """

    _DISPATCH_METHODS = ("LithoPattern",)

    @property
    def name(self):
        """str: The name of the dictionary."""
        return self._dispatch.Name

    @name.setter
    def name(self, value):
        self._dispatch.Name = value

    @property
    def nb_of_patterns(self):
        """int: The number of patterns in the dictionary."""
        return self._dispatch.NbOfPatterns

    def is_pattern(self, code):
        """Checks if the dictionary contains a pattern with the specified code.

        Parameters
        ----------
        code : str
            The code of the pattern.

        Returns
        -------
        bool
            True if successful, False otherwise.
        """
        return self._dispatch.IsPattern(code)

    def litho_pattern(self, index_or_code):
        """Gets a pattern by index or by code.

        Parameters
        ----------
        index_or_code : int or str
            The index or the code of the pattern

        Returns
        -------
        LithoPattern
            The LithoPattern object.
        """
        return LithoPattern(self._dispatch.LithoPattern(index_or_code))






from ._dispatch_wrapper import DispatchWrapper


class Depth(DispatchWrapper):
    """ The reference/master vertical axis. Can be in depth or time.

    Depth objects allow setting properties for the master depth axis
    such as scale, unit, decimals, ….

    Example
    -------
    >>> import wellcad.com
    >>> app = wellcad.com.Application()
    >>> app.new_borehole()
    <wellcad.com._borehole.Borehole object at 0x0000018B3DAF9D30>
    >>> borehole = app.get_active_borehole()
    >>> depth = borehole.depth
    """

    @property
    def decimals(self):
        """int: the number of decimals displayed in the depth string of the master depth axis."""
        return self._dispatch.Decimals

    @decimals.setter
    def decimals(self, decimals):
        self._dispatch.Decimals = decimals

    @property
    def horizontal_grid_spacing(self):
        """float: The depth grid spacing for the master depth axis"""
        return self._dispatch.HorizontalGridSpacing

    @horizontal_grid_spacing.setter
    def horizontal_grid_spacing(self, grid_spacing):
        self._dispatch.HorizontalGridSpacing = grid_spacing

    @property
    def scale(self):
        """float: The depth scale used for the master depth axis.
        
        The resulting scale is 1:``scale``.
        
        Example
        -------
        >>> depth.scale = 100 # Scale is 1:100
        """
        return self._dispatch.Scale

    @scale.setter
    def scale(self, scale):
        self._dispatch.Scale = scale

    @property
    def used_as_depth_scale(self):
        """bool: Whether this depth scale is used as the current reference axis.

        Setting this property to True on another depth log changes
        the property to False on this object. If no other depth log
        is used as the depth scale, the main depth log will
        automatically be used as the depth scale."""
        return self._dispatch.UsedAsDepthScale

    @used_as_depth_scale.setter
    def used_as_depth_scale(self, enable):
        self._dispatch.UsedAsDepthScale = enable

    @property
    def horizontal_grid(self):
        """int: The depth grid type.

        The available types are the following:

        * 0 = none
        * 1 = major grid lines only
        * 2 = major & minor grid lines
        """
        return self._dispatch.HorizontalGrid

    @horizontal_grid.setter
    def horizontal_grid(self, grid_type):
        self._dispatch.HorizontalGrid = grid_type

    @property
    def left_position(self):
        """float: The position of the left side of the master depth column as a
        fraction of the document width.
        
        In the case that this is set to be a value higher than
        ``right_position``, the two attributes will swap. Values will be
        clamped in the range [0.0, 1.0].
        """
        return self._dispatch.LeftPosition

    @left_position.setter
    def left_position(self, left_pos):
        self._dispatch.LeftPosition = left_pos

    @property
    def right_position(self):
        """float: The position of the right side of the master depth column as
        a fraction of the document width.
        
        In the case that this is set to be a value lower than
        ``left_position``, the two attributes will swap. Values will be
        clamped in the range [0.0, 1.0].
        """
        return self._dispatch.RightPosition

    @right_position.setter
    def right_position(self, right_pos):
        self._dispatch.RightPosition = right_pos

    @property
    def unit(self):
        """int: The depth unit

        The available units are the following:

        * 0 = meter
        * 1 = feet
        """
        return self._dispatch.Unit

    @unit.setter
    def unit(self, unit):
        self._dispatch.Unit = unit

    def set_position(self, left, right):
        """Sets the position and width of the master depth column.

        Values given outside the range of [0.0, 1.0] will be clamped. If
        ``left`` > ``right``, the arguments are swapped.

        Parameters
        ----------
        left : float
            The position of the left side of the depth column as a fraction of
            the document width.

        right : float
            The position of the right side of the depth column as a fraction of
            the document width.
        """
        self._dispatch.SetPosition(left, right)




from ._dispatch_wrapper import DispatchWrapper


class Workspace(DispatchWrapper):
    """Use functionalities from the Image & Structure Interpretation Workspace (ISI Workspace)
    and the Casing Integrity Workspace.

    Example
    -------
    >>> borehole = app.get_active_borehole()
    >>> workspace = borehole.workspace("ISI workspace")
    >>> workspace.representative_picks(config_file_name=config_file)
    """
    _DISPATCH_METHODS = ("PickSimilarFeatures", "RepresentativePicks", )

    def auto_detect_zones(self):
        """Automaticaly detects zones in the image log of a workspace."""
        self._dispatch.AutoDetectZones()

    def pick_similar_features(self, config_file_name):
        """Runs an automated similar structure picking in ISI workspace.

        The Pick Similar Features process takes one or more manual
        picks as input, runs the automated picking and keeps only
        picks which do not deviate from the reference(s) by a defined
        tolerance value in Dip Azimuth and Dip angle. Picks detected
        as similar are then added to the pick repository.
        You need to have **manually** selected the input picks first
        in the WellCAD window.

        Parameters
        ----------
        config_file_name : str, optional
            Path to a configuration file or a parameter string. The
            configuration file can contain the following options:

            .. code-block:: ini

                [PickSimilarFeatures]
                InputLog = (title of the log to process)
                TopDepth = 0 '(top of the interval to process)
                BottomDepth = 10 '(top of the interval to process)
                ImageType = 3 '(0 = RGB OTV, 1 = Greyscale OTV, 2 = Diamond Drilled ATV, 3 = RC Drilled ATV, 4 = FMI)
                DarkFeatures = yes /no
                BrightFeatures = yes / no
                Aperture = yes / no
                PartialPicks = yes / no
                AzimuthTolerance = 5.0
                TiltTolerance = 5.0 '(i.e. dip angle tolerance)
                UseDefaultSettings = yes / no

                ' Processing Paraemters
                ImageWidth = 144
                ImageSmoothingIterations = 2
                MinimumFeatureSize = 5
                DesiredFeatureStrength = 0.5
                MinimumFeatureStrength = 0.2
                MinimumStructureAperture = 0.008
                MaximumStructureAperture = 0.032
                SinusoidalDeviationTolerance = 30
                MaximumSinusoidAmplitude = 0.6
                MinimumSinusoidAmplitude = 0

                ' Advanced Processing Parameters
                FeatureDistanceDeviation = 2
                FeatureOrientationDeviation = 20
                HandleOverlappingSinusoids = 1 '(1 = yes, 0 = no)
                FeatureMergingConstraint = 0.6
                SinusoidCompleteness = 1 '(1 = yes, 0 = no)
                ClusterDipTolerance = 15
                ClusterAzimuthTolerance = 15
                DepthOffsetTolerance = 30
                PixelBrightnessTolerance = 1
                ImageSmoothingStrength = 20
                ImageSmoothingSpeed = 0.25
                ImageContrastSensitivity = 0.05

        """
        self._dispatch.PickSimilarFeatures(config_file_name)

    def representative_picks(self, config_file_name):
        """Runs the representative picks process in ISI

        The representative Picks process uses an adapted Ward
        algorithm to cluster picks which have the least dissimilarity
        in azimuth, dip, depth distance from each other and pick type
        (i.e. classification). As a result it determines the most
        representative pick of each cluster - i.e. the (real) pick
        with the highest confidence and closest to the center of each
        cluster.

        Parameters
        ----------
        config : str, optional
            Path to a configuration file or a parameter string. The
            configuration file can contain the following options:

            .. code-block:: ini

                [RepresentativePicks]
                InputLog = '(title of the log to process)
                TopDepth = 0 '(top of the interval to process)
                BottomDepth = 10 '(top of the interval to process)
                AzimuthWindow = 5.0
                TiltWindow = 5.0'(i.e. dip angle window)
                DepthWindow = 0.5
                KeepFeaturesUngrouped = yes / no
        """
        self._dispatch.RepresentativePicks(config_file_name)

    def automatic_picking(self, config_file_name):
        """Runs an automated structure picking in the ISI workspace

        The Auto Picker allows the detection of planar structures in
        televiewer (ATV, OTV) and electrical imager (e.g. FMI) images.
        3D core scans are similar to OTV images and can be processed
        as well.

        Parameters
        ----------
        config : str, optional
            Path to a configuration file or a parameter string. The
            configuration file can contain the following options:

            .. code-block:: ini

                [AutomaticPicking]
                InputLog = Amplitude '(title of the log to process)
                TopDepth = 0 '(top of the interval to process)
                BottomDepth = 10 '(top of the interval to process)
                ImageType = 3 '(0 = RGB OTV, 1 = Greyscale OTV, 2 = Diamond Drilled ATV, 3 = RC Drilled ATV, 4 = FMI)
                DarkFeatures = yes /no
                BrightFeatures = yes / no
                Aperture = yes / no
                PartialPicks = yes / no
                UseDefaultSettings = yes / no

                ' Processing Parameters
                ImageWidth = 144
                ImageSmoothingIterations = 2
                MinimumFeatureSize = 5
                DesiredFeatureStrength = 0.5
                MinimumFeatureStrength = 0.2
                MinimumStructureAperture = 0.008
                MaximumStructureAperture = 0.032
                SinusoidalDeviationTolerance = 30
                MaximumSinusoidAmplitude = 0.6
                MinimumSinusoidAmplitude = 0

                ' Advanced Processing Parameters
                FeatureDistanceDeviation = 2
                FeatureOrientationDeviation = 20
                HandleOverlappingSinusoids = 1 '(1 = yes, 0 = no)
                FeatureMergingConstraint = 0.6
                SinusoidCompleteness = 1 '(1 = yes, 0 = no)
                ClusterDipTolerance = 15
                ClusterAzimuthTolerance = 15
                DepthOffsetTolerance = 30
                PixelBrightnessTolerance = 1
                ImageSmoothingStrength = 20
                ImageSmoothingSpeed = 0.25
                ImageContrastSensitivity = 0.05
        """
        self._dispatch.AutomaticPicking(config_file_name)

    def quick_pick(self, config_file_name):
        """Runs a quick pick in ISI

        The Quick Pick Process analyzes the image for features with
        the highest confidence within given Azimuth, Dip and Depth
        tolerances. E.g. if you would like to pick just the main event
        every 1m interval and leave all other features unaccounted you
        can use this process.

        Parameters
        ----------
        config_file_name : str, optional
            Path to a configuration file or a parameter string. The
            configuration file can contain the following options:

            .. code-block:: ini

                [QuickPick]
                InputLog = Amplitude '(title of the log to process)
                TopDepth = 0 '(top of the interval to process)
                BottomDepth = 10 '(top of the interval to process)
                ImageType = 3 '(0 = RGB OTV, 1 = Greyscale OTV, 2 = Diamond Drilled ATV, 3 = RC Drilled ATV, 4 = FMI)
                DarkFeatures = yes /no
                BrightFeatures = yes / no
                Aperture = yes / no
                PartialPicks = yes / no
                AzimuthWindow = 5.0
                TiltWindow = 5.0'(i.e. dip angle window)
                DepthWindow = 0.5
                UseDefaultSettings = yes / no

                ' Processing Parameters
                ImageWidth = 144
                ImageSmoothingIterations = 2
                MinimumFeatureSize = 5
                DesiredFeatureStrength = 0.5
                MinimumFeatureStrength = 0.2
                MinimumStructureAperture = 0.008
                MaximumStructureAperture = 0.032
                SinusoidalDeviationTolerance = 30
                MaximumSinusoidAmplitude = 0.6
                MinimumSinusoidAmplitude = 0

                ' Advanced Processing Parameters
                FeatureDistanceDeviation = 2
                FeatureOrientationDeviation = 20
                HandleOverlappingSinusoids = 1 '(1 = yes, 0 = no)
                FeatureMergingConstraint = 0.6
                SinusoidCompleteness = 1 '(1 = yes, 0 = no)
                ClusterDipTolerance = 15
                ClusterAzimuthTolerance = 15
                DepthOffsetTolerance = 30
                PixelBrightnessTolerance = 1
                ImageSmoothingStrength = 20
                ImageSmoothingSpeed = 0.25
                ImageContrastSensitivity = 0.05
        """

        self._dispatch.QuickPick(config_file_name)

    def apply_template(self, path, prompt_if_not_found=None, config_file_name=None):
        """Applies a workspace template
        
        Allows the application of an ISI Workspace,
        Casing Integrity Workspace or NMR Workspace template.

        Parameters
        ----------
        path: str
            String specifying the name and path of the
            workspace template to be applied (\*.ist). Please note
            that the file extension is different for the
            ISI, Casing Integrity and NMR Workspaces.
        prompt_if_not_found: bool, optional
            Boolean indicating whether the dialog box to
            prompt the user for logs not found in the
            template is displayed or not. The default is
            set to True.
        config_file_name : str, optional
            The configuration file. Read the WellCAD Help for
            information on the syntax of the file.

        Returns
        -------
        bool
            Always returns True except for the WellCAD Reader and
            Demo version for which the function returns False.
        """
        return self._dispatch.ApplyTemplate(path, prompt_if_not_found, config_file_name)

    def auto_joint_detection(self, config_file_name):
        """Detects the joints from the main log (data source) used
        in the workspace.

        Parameters
        ----------
        config_file_name : str, optional
            Path to a configuration file or a parameter string. The
            configuration file can contain the following options:

            Note: This configuration is for WellCAD version 5.7 and onwards. For prior version, see the next section.

            .. code-block:: ini

                [AutoJointDetection]
                Sensitivity = 5
                TopAndBottom = yes

            For WellCAD version 5.5 and 5.6, use this configuration instead:

            .. code-block:: ini

                [AutoJointDetection]
                JointHeight=0.5
                Sensitivity=100
        """
        self._dispatch.AutoJointDetection(config_file_name)

    def add_joint_log_to_b_hole(self):
        """Adds the Marker Log from the Casing Integrity
        Workspace displaying the casing joints back to the
        borehole document.
        """
        self._dispatch.AddJointLogToBHole()

    def add_engin_log_from_driller_casing_table_to_b_hole(self):
        """Adds the Engineering Log from the Navigation Bar
        of the Casing Integrity Workspace displaying the
        drillers casing info back to the borehole document.

        Note: this function is deprecated in WellCAD 5.7 and onwards
        """
        self._dispatch.AddEnginLogFromDrillerCasingTableToBHole()

    def add_engin_log_from_logger_casing_table_to_b_hole(self):
        """Adds the Engineering Log from the Navigation Bar
        of the Casing Integrity Workspace displaying the
        casing info derived from the logged data back to the
        borehole document.
        """
        self._dispatch.AddEnginLogFromLoggerCasingTableToBHole()



from ._dispatch_wrapper import DispatchWrapper
from ._font import Font


class Title(DispatchWrapper):
    """Represents the title of a log.

    A title object is represented by a box at the top of a log. Inside the title box are three sections: the title, the comments and the properties.
    For each of these sections, you can specify properties such as text, font, color, alignment etc. The title box frame and background can also be customized.

    ::

        |-------------------|-----------------------------|-------------------|
        |                   |                             |                   |
        |     Title         |           Title             |                   |
        |                   |                             |                   |
        |                   |          comment            |      Title        |
        |                   |                             |                   |
        |    properties     |         properties          |                   |
        |-------------------|-----------------------------|-------------------|
        |                   |                             |                   |
        |                   |                             |                   |
        |    log data       |         log data            |     log data      |
        |                   |                             |                   |

    Example
    -------
    >>> borehole = app.get_active_borehole()
    >>> title = borehole.title("GR")
    >>> title.left_position
    0.2
    >>> title.box_height = 200
    """

    @property
    def left_position(self):
        """float: The position of the left side of a title for a log or group,
        as a fraction of the document width.

        If set for a group title, the logs that are a part of the group will
        be moved/scaled accordingly. In the case that this is set to be a value
        higher than ``right_position``, the two attributes will swap. Values
        will be clamped in the range [0.0, 1.0].
        """
        return self._dispatch.LeftPosition

    @left_position.setter
    def left_position(self, value):
        self._dispatch.LeftPosition = value

    @property
    def right_position(self):
        """float: The position of the right side of a title for a log or group,
        as a fraction of the document width.

        If set for a group title, the logs that are a part of the group will
        be moved/scaled accordingly. In the case that this is set to be a value
        lower than ``left_position``, the two attributes will swap. Values
        will be clamped in the range [0.0, 1.0].
        """
        return self._dispatch.RightPosition

    @right_position.setter
    def right_position(self, value):
        self._dispatch.RightPosition = value

    @property
    def box_height(self):
        """float: The title box height in mm."""
        return self._dispatch.BoxHeight

    @box_height.setter
    def box_height(self, value):
        self._dispatch.BoxHeight = value

    @property
    def display_frame(self):
        """bool: Whether the title box frame is displayed."""
        return self._dispatch.DisplayFrame

    @display_frame.setter
    def display_frame(self, flag):
        """Set to True to display the title box frame."""
        self._dispatch.DisplayFrame = flag

    @property
    def frame_color(self):
        """int: The color of the frame.

        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values.
        """
        return self._dispatch.FrameColor

    @frame_color.setter
    def frame_color(self, value):
        self._dispatch.FrameColor = value

    @property
    def frame_style(self):
        """int: The frame's display style.

        Available styles are:

        * 0 = solid
        * 1 = dashed
        * 2 = dotted
        * 3 = dash-dot
        * 4 =  dash-dot-dot
        """
        return self._dispatch.FrameStyle

    @frame_style.setter
    def frame_style(self, style):
        self._dispatch.FrameStyle = style

    @property
    def frame_width(self):
        """int: The line width of the frame in 1/10 mm."""
        return self._dispatch.FrameWidth

    @frame_width.setter
    def frame_width(self, value):
        self._dispatch.FrameWidth = value

    @property
    def background_color(self):
        """int: The background color of a group or log title.

        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values.
        """
        return self._dispatch.BackgroundColor

    @background_color.setter
    def background_color(self, value):
        self._dispatch.BackgroundColor = value

    @property
    def use_colored_background(self):
        """bool: Whether the colored background is used."""
        return self._dispatch.UseColoredBackground

    @use_colored_background.setter
    def use_colored_background(self, flag):
        self._dispatch.UseColoredBackground = flag

    @property
    def display_title(self):
        """bool: Whether the title partition is displayed."""
        return self._dispatch.DisplayTitle

    @display_title.setter
    def display_title(self, flag):
        self._dispatch.DisplayTitle = flag

    @property
    def display_comment(self):
        """bool: Whether the comment partition is displayed."""
        return self._dispatch.DisplayComment

    @display_comment.setter
    def display_comment(self, flag):
        self._dispatch.DisplayComment = flag

    @property
    def display_properties(self):
        """bool: Whether the property partition is displayed."""
        return self._dispatch.DisplayProperties

    @display_properties.setter
    def display_properties(self, flag):
        self._dispatch.DisplayProperties = flag

    @property
    def title_text(self):
        """str: The text for the title partition."""
        return self._dispatch.TitleText

    @title_text.setter
    def title_text(self, text):
        self._dispatch.TitleText = text

    @property
    def comment_text(self):
        """str: The text for the comment partition."""
        return self._dispatch.CommentText

    @comment_text.setter
    def comment_text(self, text):
        self._dispatch.CommentText = text

    @property
    def title_top(self):
        """float: The top of the title partition of a group or log
        title box as a fraction of the box height (value between 0 and 1)."""
        return self._dispatch.TitleTop

    @title_top.setter
    def title_top(self, value):
        self._dispatch.TitleTop = value

    @property
    def title_bottom(self):
        """float: The bottom of the title partition of a group or log
        title box as a fraction of the box height (value between 0 and 1)."""
        return self._dispatch.TitleBottom

    @title_bottom.setter
    def title_bottom(self, value):
        self._dispatch.TitleBottom = value

    @property
    def title_color(self):
        """int: The font color of the title part of a group or log
        title.

        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values.
        """
        return self._dispatch.TitleColor

    @title_color.setter
    def title_color(self, value):
        self._dispatch.TitleColor = value

    @property
    def comment_top(self):
        """float: The top of the comment partition of a group or log
        title box as a fraction of the box height (value between 0
        and 1)."""
        return self._dispatch.CommentTop

    @comment_top.setter
    def comment_top(self, value):
        self._dispatch.CommentTop = value

    @property
    def comment_bottom(self):
        """float: The bottom of the comment partition of a group or
        log title box as a fraction of the box height (value between 0
        and 1)."""
        return self._dispatch.CommentBottom

    @comment_bottom.setter
    def comment_bottom(self, value):
        self._dispatch.CommentBottom = value

    @property
    def comment_color(self):
        """int: The coment color of the title part of a group or log
        title.

        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values.
        """
        return self._dispatch.CommentColor

    @comment_color.setter
    def comment_color(self, value):
        self._dispatch.CommentColor = value

    @property
    def properties_top(self):
        """float: The top of the properties partition of a log title
        box as a fraction of the box height (value between 0 and 1).
        """
        return self._dispatch.PropertiesTop

    @properties_top.setter
    def properties_top(self, value):
        self._dispatch.PropertiesTop = value

    @property
    def properties_bottom(self):
        """float: The bottom of the properties partition of a log title
        box as a fraction of the box height (value between 0 and 1).
        """
        return self._dispatch.PropertiesBottom

    @properties_bottom.setter
    def properties_bottom(self, value):
        self._dispatch.PropertiesBottom = value

    @property
    def properties_color(self):
        """int: The font color of the property part of a group or log
        title.

        Colours are specified as a 32 bit integer with an ``xBGR`` structure.
        Each of the blue (B), green (G) and red (R) components are 8 bit
        values.
        """
        return self._dispatch.PropertiesColor

    @properties_color.setter
    def properties_color(self, value):
        self._dispatch.PropertiesColor = value

    @property
    def title_horizontal_position(self):
        """int: Index specifying the horizontal alignment of the title.

        Available options are:

        * 0 = left
        * 1 = center
        * 2 = right
        """
        return self._dispatch.TitleHorizontalPosition

    @title_horizontal_position.setter
    def title_horizontal_position(self, index):
        self._dispatch.TitleHorizontalPosition = index

    @property
    def title_vertical_position(self):
        """int: Index specifying the vertical alignment of the title.

        Available options are:

        * 0 = left
        * 1 = center
        * 2 = right
        """
        return self._dispatch.TitleVerticalPosition

    @title_vertical_position.setter
    def title_vertical_position(self, index):
        self._dispatch.TitleVerticalPosition = index

    @property
    def title_orientation(self):
        """int: Index specifying the title orientation.

        Available options are:

        * 0 = horizontal
        * 1 = rotated left
        * 2 = reversed
        * 3 = rotated right
        """
        return self._dispatch.TitleOrientation

    @title_orientation.setter
    def title_orientation(self, index):
        self._dispatch.TitleOrientation = index

    @property
    def comment_horizontal_position(self):
        """int: Index specifying the horizontal alignment of the comment.

        Available options are:

        * 0 = left
        * 1 = center
        * 2 = right
        """
        return self._dispatch.CommentHorizontalPosition

    @comment_horizontal_position.setter
    def comment_horizontal_position(self, index):
        self._dispatch.CommentHorizontalPosition = index

    @property
    def comment_vertical_position(self):
        """int: Index specifying the vertical alignment of the comment.

        Available options are:

        * 0 = left
        * 1 = center
        * 2 = right
        """
        return self._dispatch.CommentVerticalPosition

    @comment_vertical_position.setter
    def comment_vertical_position(self, index):
        self._dispatch.CommentVerticalPosition = index

    @property
    def comment_alignment(self):
        """int: Index specifying the comment alignment within the
        partition.

        Available options are:

        * 0 = left
        * 1 = center
        * 2 = right
        """
        return self._dispatch.CommentAlignment

    @comment_alignment.setter
    def comment_alignment(self, index):
        self._dispatch.CommentAlignment = index

    @property
    def comment_orientation(self):
        """int: Index specifying the comment orientation.

        Available options are:

        * 0 = horizontal
        * 1 = rotated left
        * 2 = reversed
        * 3 = rotated right
        """
        return self._dispatch.CommentOrientation

    @comment_orientation.setter
    def comment_orientation(self, index):
        self._dispatch.CommentOrientation = index

    @property
    def title_font(self):
        """Font: The font of the title partition of a group or log
        title."""
        return Font(self._dispatch.TitleFont)

    @property
    def comment_font(self):
        """Font: The font of the comment partition of a group or log
        title."""
        return Font(self._dispatch.CommentFont)

    @property
    def properties_font(self):
        """Font: The font of the properties  partition of a group or log
        title."""
        return Font(self._dispatch.PropertiesFont)



from ._dispatch_wrapper import DispatchWrapper


class LithoPattern(DispatchWrapper):
    """Represents a lithological pattern. These are patterns within a litho dictionary (LithoDictionary).

    You can find more information on your defined lithologies in your LithCAD library.

    Example
    -------
    >>> log = borehole.log("Lithology")
    >>> dictionary = log.litho_dictionary
    >>> pattern = dictionary.litho_pattern(0)
    >>> pattern.code
    'Sst'
    """

    @property
    def code(self):
        """str: The lithological code of the pattern.

        You can use the LithCAD tool to see your lithological
        symbols and associated codes."""
        return self._dispatch.Code

    @property
    def description(self):
        """str: The description ot the pattern."""
        return self._dispatch.Description

    @property
    def width(self):
        """int: The width of the pattern (in 1/10 mm)."""
        return self._dispatch.Width

    @property
    def height(self):
        """int: The height of the pattern (in 1/10 mm)."""
        return self._dispatch.Height

    @property
    def repeatable(self):
        """bool: The option of the symbol to be repeated
        to fill an entire depth interval or not.
        """
        return self._dispatch.Repeatable
